import unittest

from Products.PloneTestCase import ptc

from pmr2.oauth.interfaces import ICallbackManager
from pmr2.oauth.callback import CallbackManager


class DummyToken(object):

    def __init__(self, callback):
        self.callback = callback


class DummyConsumer(object):

    def __init__(self, domain):
        self.domain = domain


class CallbackManagerTestCase(unittest.TestCase):
    """
    Test the validation method of the default callback manager.
    """

    def setUp(self):
        self.cbm = CallbackManager()

    def test_0000_faildata(self):
        consumer = DummyConsumer(None)
        callback = 'oob'
        self.assertFalse(self.cbm.validate(None, callback))
        self.assertFalse(self.cbm.validate(consumer, None))

    def test_0010_no_domain_oob(self):
        consumer = DummyConsumer(None)
        callback = 'oob'
        self.assertTrue(self.cbm.validate(consumer, callback))

    def test_0011_unequal(self):
        consumer = DummyConsumer('example.com')
        callback = 'http://test.example.com/oauth_callback'
        self.assertFalse(self.cbm.validate(consumer, callback))
        # Naturally an undefined callback cannot be equal to this.
        self.assertFalse(self.cbm.validate(consumer, None))

    def test_0012_equal(self):
        consumer = DummyConsumer('example.com')
        callback = 'http://example.com/oauth_callback'
        self.assertTrue(self.cbm.validate(consumer, callback))

    def test_0100_valid_subdomain(self):
        consumer = DummyConsumer('*.example.com')
        callback = 'http://test.example.com/oauth_callback'
        self.assertTrue(self.cbm.validate(consumer, callback))

    def test_0110_invalid_subdomain(self):
        consumer = DummyConsumer('*.example.com')
        callback = 'http://testexample.com/oauth_callback'
        self.assertFalse(self.cbm.validate(consumer, callback))

        consumer = DummyConsumer('*example.com')
        callback = 'http://example.com/oauth_callback'
        self.assertFalse(self.cbm.validate(consumer, callback))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(CallbackManagerTestCase))
    return suite



from zope.publisher.browser import BrowserPage
from Products.CMFCore.utils import getToolByName 


class CurrentUserView(BrowserPage):

    def __call__(self):
        mt = getToolByName(self.context, 'portal_membership')
        if mt.isAnonymousUser():
            return 'Anonymous User'
        user = mt.getAuthenticatedMember()
        return user.id


class CurrentRolesView(BrowserPage):

    def __call__(self):
        mt = getToolByName(self.context, 'portal_membership')
        roles = '\n'.join(mt.getAuthenticatedMember().getRoles())
        return roles


class OAuthCallbackView(BrowserPage):

    def __call__(self):
        verifier = 'Verifier: %s' % self.request.form.get('oauth_verifier')
        token = 'Token: %s' % self.request.form.get('oauth_token')
        return verifier + '\n' + token



from zope.interface import Interface
import zope.component
from time import time
import unittest

from zExceptions import Forbidden
from zExceptions import BadRequest

from pmr2.oauth.interfaces import *

from pmr2.oauth.utility import SiteRequestOAuth1ServerAdapter

from pmr2.oauth.token import Token
from pmr2.oauth.token import TokenManager

from pmr2.oauth.consumer import Consumer
from pmr2.oauth.consumer import ConsumerManager

from pmr2.oauth.tests.base import IOAuthTestLayer
from pmr2.oauth.tests.base import TestRequest
from pmr2.oauth.tests.base import SignedTestRequest

from pmr2.oauth.tests.adapter import TestCallbackManager


def mock_factory(cls):
    instance = cls()
    def getInstance(context, request):
        return instance
    return getInstance


class TestExtraction(unittest.TestCase):

    default_consumer_key = 'consumer.example.com'
    default_user_id = 'test_user'

    def setUp(self):
        tmf = mock_factory(TokenManager)
        cmf = mock_factory(ConsumerManager)
        self.plugin = self.createPlugin()
        zope.component.provideAdapter(
            cmf, (Interface, IOAuthTestLayer,), IConsumerManager)
        zope.component.provideAdapter(
            tmf, (Interface, IOAuthTestLayer,), ITokenManager)
        zope.component.provideAdapter(
            TestCallbackManager, (Interface, IOAuthTestLayer), ICallbackManager)
        zope.component.provideAdapter(SiteRequestOAuth1ServerAdapter,
            (Interface, IOAuthTestLayer,), IOAuthAdapter)

        self.consumerManager = zope.component.getMultiAdapter(
            (object, TestRequest()), IConsumerManager)
        self.tokenManager = zope.component.getMultiAdapter(
            (object, TestRequest()), ITokenManager)
        self.callbackManager = zope.component.getMultiAdapter(
            (object, TestRequest()), ICallbackManager)

    def createPlugin(self):
        from pmr2.oauth.tests.utility import MockPAS
        from pmr2.oauth.tests.utility import MockSite
        from pmr2.oauth.plugins.oauth import OAuthPlugin

        class NoScopeOAuthPlugin(OAuthPlugin):
            def _validateScope(self, *a, **kw):
                return True

        plugin = NoScopeOAuthPlugin("oauth")
        return plugin.__of__((MockPAS()).__of__(MockSite()))

    def generate_consumer_and_token(self, consumer_key=None,
            save_consumer=False, save_token=False):
        if not consumer_key:
            consumer_key = self.default_consumer_key
        consumer = Consumer('consumer.example.com', 'consumer-secret')
        token = Token('token-key', 'token-secret')
        token.access = True
        token.user = self.default_user_id
        token.consumer_key = 'consumer.example.com'
        if save_consumer:
            self.consumerManager.add(consumer)
        if save_token:
            self.tokenManager.add(token)
        return consumer, token

    def save_consumer_and_token(self, consumer_key=None):
        return self.generate_consumer_and_token(consumer_key, True, True)

    def test_0000_empty_extraction(self):
        plugin = self.plugin
        creds = plugin.extractCredentials(plugin.REQUEST)
        self.assertEqual(creds, {})

    def test_0100_fail_missing_unsigned_token(self):
        plugin = self.plugin
        timestamp = str(int(time()))
        request = TestRequest(
            oauth_keys={
                'oauth_token': "nosuchtoken",
                'oauth_version': "1.0",
                'oauth_nonce': "fakenonce",
                'oauth_timestamp': timestamp,
            },
        )
        self.assertRaises(BadRequest, plugin.extractCredentials, request)

    def test_0200_fail_unsigned_request(self):
        plugin = self.plugin
        consumer, token = self.save_consumer_and_token()

        timestamp = str(int(time()))
        request = TestRequest(
            oauth_keys={
                'oauth_consumer_key': consumer.key,
                'oauth_token': token.key,
                'oauth_version': "1.0",
                'oauth_nonce': "fakenonce",
                'oauth_timestamp': timestamp,
            },
        )
        self.assertRaises(BadRequest, plugin.extractCredentials, request)

    def test_0201_fail_badly_signed_request(self):
        plugin = self.plugin
        consumer, token = self.save_consumer_and_token()

        timestamp = str(int(time()))
        request = TestRequest(
            oauth_keys={
                'oauth_consumer_key': consumer.key,
                'oauth_token': token.key,
                'oauth_version': "1.0",
                'oauth_nonce': "fakenonce",
                'oauth_timestamp': timestamp,
                'oauth_signature_method': "HMAC-SHA1",
                'oauth_signature': "badsignature",
                'oauth_body_hash': "2jmj7l5rSw0yVb%2FvlWAYkK/YBwk=", 
            },
        )
        self.assertRaises(BadRequest, plugin.extractCredentials, request)

    def test_0202_fail_bad_timestamp(self):
        plugin = self.plugin
        consumer, token = self.save_consumer_and_token()

        timestamp = '0'
        request = SignedTestRequest(consumer=consumer, token=token,
            timestamp=timestamp)
        self.assertRaises(BadRequest, plugin.extractCredentials, request)

    def test_0210_fail_signed_mismatch_both_secrets(self):
        plugin = self.plugin
        real_consumer, real_token = self.save_consumer_and_token()
        consumer, token = self.generate_consumer_and_token()
        consumer.secret = 'failfailfailfail'
        token.secret = 'failfailfailfail'

        request = SignedTestRequest(consumer=consumer, token=token,)
        self.assertRaises(Forbidden, plugin.extractCredentials, request)

    def test_0211_fail_signed_mismatch_consumer_secret(self):
        plugin = self.plugin
        real_consumer, real_token = self.save_consumer_and_token()
        consumer, token = self.generate_consumer_and_token()
        consumer.secret = 'failfailfailfail'
        token = real_token

        request = SignedTestRequest(consumer=consumer, token=token,)
        self.assertRaises(Forbidden, plugin.extractCredentials, request)

    def test_0212_fail_signed_mismatch_token_secret(self):
        plugin = self.plugin
        real_consumer, real_token = self.save_consumer_and_token()
        consumer, token = self.generate_consumer_and_token()
        token.secret = 'failfailfailfail'
        consumer = real_consumer

        request = SignedTestRequest(consumer=consumer, token=token,)
        self.assertRaises(Forbidden, plugin.extractCredentials, request)

    def test_0300_fail_request_token(self):
        plugin = self.plugin
        consumer, atoken = self.generate_consumer_and_token(save_consumer=True)
        # make request token
        token = self.tokenManager.generateRequestToken(consumer.key, 
            'http://callback.example.com/')

        # Request token generated properly and stored, but verifier not
        # signed and provided with request.
        request = SignedTestRequest(consumer=consumer, token=token,)
        self.assertRaises(Forbidden, plugin.extractCredentials, request)

        # Verifier provided, but no users have claimed this token.
        request = SignedTestRequest(consumer=consumer, token=token, 
            verifier=token.verifier)
        self.assertRaises(Forbidden, plugin.extractCredentials, request)

    def test_0301_success_request_token(self):
        plugin = self.plugin
        consumer, atoken = self.generate_consumer_and_token(save_consumer=True)
        # make request token
        token = self.tokenManager.generateRequestToken(consumer.key, 
            'http://callback.example.com/')

        # have a user claim this.
        self.tokenManager.claimRequestToken(token, 'user')

        # Request token generated properly and stored, with verifier
        # signed and provided with request, and a user has claimed the
        # token.
        request = SignedTestRequest(consumer=consumer, token=token, 
            verifier=token.verifier)

        # Credentials successfully not returned.
        self.assertEqual(plugin.extractCredentials(request), {})

        # Now if the generated request token was removed from the store,
        # this same token request would be forbidden.
        request = SignedTestRequest(consumer=consumer, token=token,)
        self.tokenManager.remove(token)
        self.assertRaises(Forbidden, plugin.extractCredentials, request)

        # Since atoken isn't saved either, it should have failed too.
        request = SignedTestRequest(consumer=consumer, token=atoken,)
        self.assertRaises(Forbidden, plugin.extractCredentials, request)

    def test_0500_fail_access_token_no_consumer(self):
        # use request token
        plugin = self.plugin
        consumer, token = self.generate_consumer_and_token(save_token=True)
        request = SignedTestRequest(consumer=consumer, token=token,)
        self.assertRaises(Forbidden, plugin.extractCredentials, request)

    def test_1000_success_access_token(self):
        # use request token
        plugin = self.plugin
        consumer, token = self.save_consumer_and_token()

        request = SignedTestRequest(consumer=consumer, token=token,)
        credentials = plugin.extractCredentials(request)
        self.assertEqual(credentials['userid'], self.default_user_id)

    def test_1100_missing_token_ignored(self):
        # Should not forbid cases where the oauth_token is missing (it
        # could be a RequestToken, let that page handle it).
        plugin = self.plugin
        consumer, token = self.save_consumer_and_token()

        request = SignedTestRequest(consumer=consumer, callback='oob')
        credentials = plugin.extractCredentials(request)
        self.assertEqual(credentials, {})

    def test_1101_strip_access_token_failure(self):
        # Should not forbid cases where the oauth_token is a request
        # token, as it could be used to request for an access token.
        # (at least this is a valid token, just has no credentials).
        plugin = self.plugin
        consumer, token = self.save_consumer_and_token()

        # forcibily strip that token's access rights.
        token.access = False

        request = SignedTestRequest(consumer=consumer, token=token,)
        self.assertRaises(Forbidden, plugin.extractCredentials, request)

    def test_2000_base_oauth_adapter(self):
        oauth1 = zope.component.getMultiAdapter(
            (object, TestRequest()), IOAuthAdapter)
        self.assertEqual(oauth1.dummy_request_token,
            self.tokenManager.DUMMY_KEY)
        self.assertEqual(oauth1.dummy_access_token,
            self.tokenManager.DUMMY_KEY)

        # should still work if purged.
        self.tokenManager.remove(self.tokenManager.DUMMY_KEY)
        self.assertEqual(oauth1.dummy_request_token,
            self.tokenManager.DUMMY_KEY)
        self.assertEqual(oauth1.dummy_access_token,
            self.tokenManager.DUMMY_KEY)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestExtraction))
    return suite



import os
import base64
from urllib import quote_plus

import oauthlib.oauth1
from oauthlib.common import urldecode

import zope.interface
import zope.schema
from zope.app.component.hooks import getSite

from Products.CMFCore.utils import getToolByName

from pmr2.oauth.interfaces import TokenInvalidError, ICallbackManager
from pmr2.oauth.interfaces import IOAuthAdapter, INonceManager
from pmr2.oauth.interfaces import IConsumerManager, ITokenManager

from pmr2.oauth.schema import buildSchemaInterface, CTSMMappingList

SAFE_ASCII_CHARS = set([chr(i) for i in xrange(32, 127)])


class SiteRequestOAuth1ServerAdapter(oauthlib.oauth1.Server):
    """
    Completing the implementation of the server as an adapter that
    unifies the authentication process.
    """

    zope.interface.implements(IOAuthAdapter)

    def __init__(self, site, request):
        self.consumerManager = zope.component.getMultiAdapter(
            (site, request), IConsumerManager)
        self.tokenManager = zope.component.getMultiAdapter(
            (site, request), ITokenManager)
        self.callbackManager = zope.component.getMultiAdapter(
            (site, request), ICallbackManager)
        # Optional at this point.
        self.nonceManager = zope.component.queryMultiAdapter(
            (site, request), INonceManager)

        self.access_key = None

        self.site = site
        self.request = request

        # All OAuth related terms are placed in the Authorization.
        # Using unicode because lol oauthlib's obsession with 
        # unicode_literals, which adds nothing but annoyance when the
        # protocol is fundamentally in bytes.

        self.uri = safe_unicode(extractRequestURL(request))
        self.http_method = safe_unicode(request.method)

        # These are the only headers that affect the signature for
        # an OAuth request.

        self.headers = {
            u'Content-type': safe_unicode(request.getHeader('Content-type')),
            u'Authorization': safe_unicode(request._auth),
        }

        request.stdin.seek(0)
        self.body = safe_unicode(request.stdin.read())

    # Local helpers.

    def __call__(self):
        result, request = self.verify_pas_request()
        if result:
            # Only set the access key if access was granted.
            self.client_key = request.client_key
            self.access_key = request.resource_owner_key
        # We only want a True or False value.
        return result

    def mark_request(self, oauth_request):
        """
        Mark the request object with a flag to give hints for the token
        management pages.
        """

        self.request._pmr2_oauth1_ = oauth_request

    def verify_pas_request(self):
        """
        Verify a standard request with all checks in order to not
        trigger a BadRequest or Forbidden error prematurely, which would
        prevent access to the RequestToken and AccessToken pages.
        """

        req_result = acc_result = result = None
        acc_request = req_request = request = None 

        try:
            req_result, req_request = self.verify_request_token_request()
        except ValueError:
            pass

        try:
            acc_result, acc_request = self.verify_access_token_request()
        except ValueError:
            pass

        try:
            result, request = self.verify_resource_request()
        except ValueError:
            if not (req_result or acc_result):
                raise

        if result is False:
            # Check to see if either one of those passed.  Return an
            # undefined result (as None) and whichever request object
            # that got generated.
            if req_result:
                self.mark_request(req_request)
                return None, req_request

            if acc_result:
                self.mark_request(acc_request)
                return None, acc_request

        return result, request

    def verify_resource_request(self):
        """
        For verification of requests for accessing resources.
        """

        return self.verify_request(
            require_resource_owner=True,
            require_verifier=False,
            require_callback=False,
        )

    def verify_request_token_request(self, uri=None, http_method=None,
            body=None, headers=None):
        """
        For verification of requests for getting RequestTokens.
        """

        return self.verify_request(
            uri=uri,
            http_method=http_method,
            body=body,
            headers=headers,
            require_resource_owner=False, 
            require_verifier=False,
            require_callback=True,
        )

    def verify_access_token_request(self, uri=None, http_method=None,
            body=None, headers=None):
        """
        For verification of requests for getting AccessTokens.
        """

        return self.verify_request(
            uri=uri,
            http_method=http_method,
            body=body,
            headers=headers,
            require_resource_owner=True, 
            require_verifier=True,
            require_callback=False,
        )

    # Overrides

    def verify_request(self, uri=None, http_method=None, body=None,
            headers=None, require_resource_owner=True, require_verifier=False,
            require_realm=False, required_realm=None, require_callback=False,
            *a, **kw):
        """
        Essentially a clone of the parent class, but the default 
        parameters will be from this class.
        """

        if uri is None:
            uri = self.uri
        if http_method is None:
            http_method = self.http_method
        if body is None:
            body = self.body
        if headers is None:
            headers = self.headers

        return super(SiteRequestOAuth1ServerAdapter, self).verify_request(
            uri, http_method, body, headers, 
            require_resource_owner, require_verifier, require_realm,
            required_realm, require_callback)

    # Property overrides

    @property
    def enforce_ssl(self):
        # Since the reverse proxy does not provide any ssl and any 
        # secure traffic would not be reflected here.  However, this
        # value is used to verify whether to need the https be in the
        # signing URL, so there might be a need to provide a way to
        # configure this.
        return False

    @property
    def safe_characters(self):
        return SAFE_ASCII_CHARS

    @property
    def client_key_length(self):
        return 8, 64

    @property
    def request_token_length(self):
        return 8, 64

    @property
    def access_token_length(self):
        return 8, 64

    # Not implemented but used

    @property
    def dummy_resource_owner(self):
        # This is not actually used.
        return u''

    # Implementation

    def get_client_secret(self, client_key):
        consumer = self.consumerManager.getValidated(client_key)
        # Spend actual time failing.
        dummy = self.consumerManager.getValidated(
            self.consumerManager.DUMMY_KEY)

        if consumer:
            result = consumer.secret
        else:
            result = self.consumerManager.DUMMY_SECRET

        return unicode(result)

    @property
    def dummy_client(self):
        result = self.consumerManager.DUMMY_KEY
        return unicode(result)

    def get_request_token_secret(self, client_key, request_token):
        token = self.tokenManager.getRequestToken(request_token, None)
        if token and token.consumer_key == client_key:
            result = token.secret
        else:
            result = self.tokenManager.DUMMY_SECRET

        return unicode(result)

    def get_access_token_secret(self, client_key, access_token):
        token = self.tokenManager.getAccessToken(access_token, None)
        if token and token.consumer_key == client_key:
            result = token.secret
        else:
            result = self.tokenManager.DUMMY_SECRET

        return unicode(result)

    @property
    def dummy_request_token(self):
        token = self.tokenManager.get(self.tokenManager.DUMMY_KEY)
        result = token and token.key or self.tokenManager.DUMMY_KEY

        return unicode(result)

    @property
    def dummy_access_token(self):
        token = self.tokenManager.get(self.tokenManager.DUMMY_KEY)
        result = token and token.key or self.tokenManager.DUMMY_KEY

        return unicode(result)

    def get_rsa_key(self, client_key):
        # TODO figure out how to implement this.
        raise NotImplementedError

    def validate_client_key(self, client_key):
        # This will search through the table to acquire a failed dummy key
        dummy = self.consumerManager.get(self.consumerManager.DUMMY_KEY,
            self.consumerManager.makeDummy())
        consumer = self.consumerManager.getValidated(client_key, dummy)
        return consumer.validate() and consumer != dummy

    def validate_request_token(self, client_key, request_token):
        token = self.tokenManager.getRequestToken(request_token, 
            self.dummy_request_token)
        return (token != self.dummy_request_token and 
            token.consumer_key == client_key)

    def validate_access_token(self, client_key, access_token):
        token = self.tokenManager.getAccessToken(access_token, 
            self.dummy_access_token)
        return (token != self.dummy_access_token and 
            token.consumer_key == client_key)

    def validate_timestamp_and_nonce(self, client_key, timestamp, nonce,
            request_token=None, access_token=None):
        if self.nonceManager:
            # TODO figure out the parameters.
            return self.nonceManager.validate(timestamp, nonce)
        # Just let this one go...
        return True

    def validate_redirect_uri(self, client_key, redirect_uri):
        # Redirect URI will be external, verify that it is in the
        # same format as it was registered for the consumer.
        if redirect_uri is None:
            # Most requests will not have this defined.
            return True

        # Only the token request will have this
        consumer = self.consumerManager.getValidated(client_key)
        return self.callbackManager.validate(consumer, redirect_uri)

    def validate_requested_realm(self, client_key, realm):
        # Realms are not handled.
        return True

    def validate_realm(self, client_key, access_token, uri=None,
            required_realm=None):
        # Realms are not handled.
        return True

    def validate_verifier(self, client_key, request_token, verifier):
        try:
            return self.tokenManager.requestTokenVerify(
                client_key, request_token, verifier)
        except TokenInvalidError:
            return False


def random_string(length):
    """
    Request a random string up to this length.

    This method attempts to use the OS provided random bytes 
    suitable for cryptographical use (see os.urandom), base64 
    encoded (see base64.urlsafe_b64encode).  Hence actual length
    will be divisible by 4.
    """

    actual = int(length / 4) * 3
    return base64.urlsafe_b64encode(os.urandom(actual))

def extractRequestURL(request):
    # I am not sure why there isn't a thing that gets me the original
    # URI in the HTTP header and has to reconstruct all of this from
    # broken up pieces.

    actual_url = request.get('ACTUAL_URL', None)
    query_string = request.get('QUERY_STRING', None)
    
    if actual_url:
        result = actual_url
        if query_string:
            # XXX this is _seriously_ a pita.  For some reason in some
            # circumstances we are not getting back the original encoded
            # values that we used to sign the request.  This rectifies
            # that.
            try:
                urldecode(query_string)
            except ValueError:
                query_string = quote_plus(query_string, safe='=&;%+~')
            result += '?' + query_string
    else:
        # fallback.
        result = request.getURL()

    return result

def safe_unicode(s):
    if isinstance(s, str):
        return unicode(s)
    return s
   
def getUserPortalTypes():
    site = getSite()
    portal_catalog = getToolByName(site, 'portal_catalog')
    plone_utils = getToolByName(site, 'plone_utils')
    portal_types = getToolByName(site, 'portal_types')
    all_used_types = portal_catalog.uniqueValuesFor('portal_type');
    all_friendly_types = plone_utils.getUserFriendlyTypes(all_used_types)
    # XXX assumption, should use portal_types.getTypeInfo
    root = [('Plone Site', 'Plone Site')]  
    main = [(t, portal_types.getTypeInfo(t).title) for t in all_friendly_types]
    return root + sorted(main, lambda x, y: cmp(x[1], y[1]))

def schemaFactory(**kw):
    title = unicode(kw.pop('title'))
    required = kw.pop('required')
    return CTSMMappingList(title=title, required=required,
        value_type=zope.schema.ASCIILine())

def buildContentTypeScopeProfileInterface():
    types = getUserPortalTypes()
    return buildSchemaInterface(types, schemaFactory, sort=False)



import time
import urlparse

from persistent import Persistent
from persistent.list import PersistentList
from BTrees.OOBTree import OOBTree

from zope.app.container.contained import Contained
from zope.annotation.interfaces import IAttributeAnnotatable

import zope.interface
from zope.schema import fieldproperty

from pmr2.oauth.interfaces import IToken
from pmr2.oauth.interfaces import ITokenManager
from pmr2.oauth.interfaces import CallbackValueError
from pmr2.oauth.interfaces import TokenInvalidError, ExpiredTokenError
from pmr2.oauth.interfaces import NotAccessTokenError, NotRequestTokenError
from pmr2.oauth.factory import factory
from pmr2.oauth.utility import random_string


class TokenManager(Persistent, Contained):
    """\
    A basic token manager for the default layer.

    This manager provides the bare minimum functionality, currently it
    does not easily provide a way for users to check what tokens they
    have approved.
    """

    zope.component.adapts(IAttributeAnnotatable, zope.interface.Interface)
    zope.interface.implements(ITokenManager)

    DUMMY_KEY = 'dummy'
    DUMMY_SECRET = 'dummy'

    # expiry
    claim_timeout = 180
    
    def __init__(self):
        self._tokens = OOBTree()
        self._user_token_map = OOBTree()
        dummy = self._makeDummy()
        self.add(dummy)

    def _makeDummy(self):
        dummy = Token(self.DUMMY_KEY, self.DUMMY_SECRET)
        return dummy

    def _add_user_map(self, token):
        if not token.access or token.user is None:
            return

        # only tracking access tokens with user defined.
        user_tokens = self._user_token_map.get(token.user, None)
        if user_tokens is None:
            user_tokens = PersistentList()
            self._user_token_map[token.user] = user_tokens

        user_tokens.append(token.key)

    def _del_user_map(self, token):
        if token.user is None:
            return

        # only tracking access tokens with user defined.
        user_tokens = self._user_token_map.get(token.user, None)
        if user_tokens is None:
            # guess this user didn't have any tokens tracked before.
            return

        if token.key in user_tokens:
            # Well this key may not have been mapped.
            user_tokens.remove(token.key)

    def add(self, token):
        assert IToken.providedBy(token)
        if self.get(token.key):
            raise ValueError('token %s already exists', token.key)
        self._tokens[token.key] = token
        self._add_user_map(token)

    def _generateBaseToken(self, consumer_key):
        key = random_string(24)
        secret = random_string(24)
        token = Token(key, secret)
        token.consumer_key = consumer_key
        token.timestamp = int(time.time())
        return token

    def generateRequestToken(self, consumer_key, callback):
        """\
        Generate request token from consumer and request.
        """

        # This is our constraint.
        if callback is None:
            raise CallbackValueError(
                'callback must be specified or set to `oob`')

        token = self._generateBaseToken(consumer_key)
        token.set_callback(callback)
        token.set_verifier()

        token.expiry = int(time.time()) + self.claim_timeout

        self.add(token)
        return token

    def generateAccessToken(self, consumer_key, request_token):

        # Get the copy that is being tracked here.
        old_token = self.get(request_token)
        if not old_token:
            raise TokenInvalidError('invalid token')
        old_key = old_token.key

        token = self._generateBaseToken(consumer_key)
        token.access = True

        # Must have a user.
        if not old_token.user:
            raise TokenInvalidError('token has no user')
        token.user = old_token.user

        if old_token:
            # Terminate old token to prevent reuse.
            self.remove(old_key)

        # Now add token.
        self.add(token)
        return token

    def claimRequestToken(self, token, user):
        token = self.get(token)
        if not token:
            raise TokenInvalidError('invalid token')
        if token.access:
            raise TokenInvalidError('not request token')
        token.user = user
        token.expiry = int(time.time()) + self.claim_timeout

    def get(self, token, default=None):
        token_key = IToken.providedBy(token) and token.key or token
        return self._tokens.get(token_key, default)

    def getRequestToken(self, token, default=False):
        token = self.get(token, default)
        if token is default:
            if default is False:
                raise TokenInvalidError('no such request token.')
            return default

        if token.access:
            if default is False:
                raise NotRequestTokenError('not a request token.')
            return default

        return token

    def getAccessToken(self, token, default=False):
        token = self.get(token, default)
        if token is default:
            if default is False:
                raise TokenInvalidError('no such access token.')
            return default

        if not token.access:
            if default is False:
                raise NotAccessTokenError('not an access token.')
            return default

        # must be identified
        if not token.user:
            raise TokenInvalidError('token has no user')
        raw_keys = self._user_token_map.get(token.user, [])
        if token.key not in raw_keys:
            raise TokenInvalidError('user `%s` does not own this key' 
                % token.user)

        return token

    def getTokensForUser(self, user):
        raw_keys = self._user_token_map.get(user, [])
        result = [self.get(t) for t in raw_keys]
        return result

    def remove(self, token):
        if IToken.providedBy(token):
            token = token.key
        token = self._tokens.pop(token)
        self._del_user_map(token)
        return token

    def requestTokenVerify(self, consumer_key, token, verifier):
        """\
        Verify that the request results in a valid token by checking for
        validity of the consumer_key and verifier.
        """

        token = self.getRequestToken(token)
        return (token.consumer_key == consumer_key and 
                token.verifier == verifier and
                token.user is not None
                )

TokenManagerFactory = factory(TokenManager)


class Token(Persistent):

    zope.interface.implements(IToken)

    key = fieldproperty.FieldProperty(IToken['key'])
    secret = fieldproperty.FieldProperty(IToken['secret'])
    callback = fieldproperty.FieldProperty(IToken['callback'])
    verifier = fieldproperty.FieldProperty(IToken['verifier'])
    access = fieldproperty.FieldProperty(IToken['access'])

    user = fieldproperty.FieldProperty(IToken['user'])
    consumer_key = fieldproperty.FieldProperty(IToken['consumer_key'])
    timestamp = fieldproperty.FieldProperty(IToken['timestamp'])
    expiry = fieldproperty.FieldProperty(IToken['expiry'])

    def __init__(self, key, secret):
        assert not ((key is None) or (secret is None))
        self.key = key
        self.secret = secret

    def set_callback(self, callback):
        """
        Set the callback URI.
        """

        self.callback = callback

    def set_verifier(self, verifier=None):
        """
        Based on implementation from python-oauth2
        
        This uses the random_string method to generate the verifier key.
        Does not regenerate a new key if verifier is already set, unless
        a specific verifier is specified which will the be set.  To
        generate a new random value, set the verifier parameter to True.
        """

        if verifier is True or (verifier is None and self.verifier is None):
            # As the user could hit back or whatever reason and reload
            # the key before the consumer gets to complete the request,
            # and to simplify how this is invoked by the authorization
            # form, we only generate this once, unless the caller really
            # wants to do this.
            self.verifier = random_string(24)
            return

        if verifier is not None:
            # fieldproperty should validate this input automatically.
            self.verifier = verifier

    def get_callback_url(self):
        """
        Based on implementation from python-oauth2, with corrections.
        """

        if self.callback and self.verifier:
            # Append the oauth_verifier.
            parts = urlparse.urlparse(self.callback)
            scheme, netloc, path, params, query, fragment = parts[:6]
            q = query and [query] or []
            q.append('oauth_verifier=%s' % self.verifier)
            q.append('oauth_token=%s' % self.key)
            query = '&'.join(q)
            return urlparse.urlunparse((scheme, netloc, path, params,
                query, fragment))
        return self.callback



import urlparse

import zope.component
import zope.interface

from pmr2.oauth.interfaces import ICallbackManager


class CallbackManager(object):
    """
    Default callback manager.
    """

    zope.interface.implements(ICallbackManager)

    def __init__(self, *a, **kw):
        pass

    def validate(self, consumer, callback):
        """
        Check whether callback is valid for this consumer.
        """

        if not consumer:
            return False

        domain = consumer.domain

        if callback == 'oob':
            # out of band is always valid.
            return True

        if not callback or not domain:
            return False

        parts = urlparse.urlparse(callback)
        scheme, netloc, path, params, query, fragment = parts[:6]

        # verify netloc against consumer's domain.

        if isinstance(domain, basestring) and domain.startswith('*.'):
            return netloc.endswith(domain[1:])

        return netloc == domain



import zope.component
import zope.interface
from zope.app.component.hooks import getSite
from zope.publisher.browser import BrowserView
from zope.publisher.interfaces import NotFound
from zope.publisher.interfaces import IPublishTraverse

from Acquisition import Implicit
from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage

from z3c.form import field
from z3c.form import button
from z3c.form.interfaces import DISPLAY_MODE

from pmr2.z3cform import form
from pmr2.z3cform import page

from pmr2.oauth import MessageFactory as _
from pmr2.oauth.interfaces import IScopeManager, IContentTypeScopeManager
from pmr2.oauth.interfaces import IContentTypeScopeProfile
from pmr2.oauth.interfaces import _IDynamicSchemaInterface
from pmr2.oauth.scope import ContentTypeScopeProfile

from pmr2.oauth.browser.interfaces import IContentTypeScopeProfileAdd
from pmr2.oauth.browser.interfaces import IContentTypeScopeProfileEdit
from pmr2.oauth.browser.template import path, ViewPageTemplateFile


class ScopeEditForm(form.EditForm):
    """\
    For editing scope.
    """

    @property
    def fields(self):
        sm = self.getContent()
        # this assumes the scope manager implements the immediate 
        # interface and that it provides all the fields for it.
        inf = zope.component.providedBy(sm).interfaces().next()
        return field.Fields(inf)

    def update(self):
        super(ScopeEditForm, self).update()
        self.request['disable_border'] = True

    def getContent(self):
        sm = zope.component.getMultiAdapter(
            (self.context, self.request), IContentTypeScopeManager)
        return sm


class ContentTypeScopeManagerView(Implicit, page.SimplePage):

    template = ViewPageTemplateFile(path('ctsm_view.pt'))

    def update(self):
        self.request['disable_border'] = True

    @property
    def label(self):
        return _(u'Content Type Scope Manager')

    @property
    def url_expr(self):
        # URL expression for this view.
        return '%s/%s' % (self.context.absolute_url(), self.__name__)


    def getContent(self):
        sm = zope.component.getMultiAdapter(
            (self.context, self.request), IContentTypeScopeManager)
        return sm

    def getProfileNames(self):
        """
        Return a unified list of the active profiles and WIP ones.
        """

        sm = self.getContent()
        pn = sm.getMappingNames()
        epn = sm.getEditProfileNames()
        # could yield this value, but it's small so save effort and be
        # naive.
        return sorted(list(set(list(pn) + list(epn))))

    def publishTraverse(self, request, name):
        # Since this is registerd as a view and is Implicit, it needs
        # this or this doesn't get resolved.
        return self


class ContentTypeScopeProfileTraverseForm(form.AuthenticatedForm,
        page.TraversePage):
    # TODO split the form specific part up.

    def _getProfileAndMapping(self):
        profile = self.getContent()
        site = getSite()
        sm = zope.component.getMultiAdapter(
            (site, self.request), IContentTypeScopeManager)
        mapping = sm.getMappingByName(self.profile_name, default={})
        return profile, mapping

    @property
    def profile_name(self):
        if not self.traverse_subpath or len(self.traverse_subpath) > 1:
            return None

        return self.traverse_subpath[0]

    def update(self):
        self.request['disable_border'] = True
        super(ContentTypeScopeProfileTraverseForm, self).update()

    def getContent(self):
        # only do this once.
        if not hasattr(self, '_content'):
            self._source = self.getSource()
        return self._source

    def getSource(self):
        name = self.profile_name
        if not name:
            raise NotFound(self.context, '')

        site = getSite()
        sm = zope.component.getMultiAdapter(
            (site, self.request), IContentTypeScopeManager)
        obj = sm.getEditProfile(name)
        if obj is None:
            raise NotFound(self.context, name)
        return obj


class ContentTypeScopeProfileDisplayForm(ContentTypeScopeProfileTraverseForm):

    # May just take a shortcut and have a custom template to show icons.
    #template = ViewPageTemplateFile(path('ctsp_view.pt'))

    fields = field.Fields(IContentTypeScopeProfileEdit)
    mode = DISPLAY_MODE
    next_target = None

    @button.buttonAndHandler(_('Edit'), name='edit')
    def handleEdit(self, action):
        self.authenticate()
        if self.profile_name:
            # absolute_url is implicitly acquired by the parent view.
            self.next_target = '/'.join([self.context.absolute_url(), 
                self.context.__name__, 'edit', self.profile_name,])

    @button.buttonAndHandler(_('Commit Update'), name='commit')
    def handleCommit(self, action):
        self.authenticate()
        site = getSite()
        sm = zope.component.getMultiAdapter(
            (site, self.request), IContentTypeScopeManager)
        sm.commitEditProfile(self.profile_name)

        self.next_target = '/'.join([self.context.absolute_url(), 
            self.context.__name__, 'view', self.profile_name,])

    @button.buttonAndHandler(_('Revert'), name='revert')
    def handleRevert(self, action):
        self.authenticate()
        profile, original = self._getProfileAndMapping()
        profile.mapping = original

        self.next_target = '/'.join([self.context.absolute_url(), 
            self.context.__name__, 'view', self.profile_name,])

    @button.buttonAndHandler(_('Set as Default'), name='setdefault')
    def handleSetDefault(self, action):
        self.authenticate()
        site = getSite()
        sm = zope.component.getMultiAdapter(
            (site, self.request), IContentTypeScopeManager)
        try:
            sm.default_mapping_id = sm.getMappingId(self.profile_name)
        except KeyError:
            status = IStatusMessage(self.request)
            status.addStatusMessage(_(
                u'This profile has not been committed yet.'),
                'error'
            )

        self.next_target = '/'.join([self.context.absolute_url(), 
            self.context.__name__, 'view', self.profile_name,])

    def update(self):
        super(ContentTypeScopeProfileDisplayForm, self).update()

        if self.isMappingModified():
            status = IStatusMessage(self.request)
            status.addStatusMessage(_(
                u'This profile has been modified.  Please commit the changes '
                 'when they are ready.'),
                'info'
            )

        if self.next_target:
            self.request.response.redirect(self.next_target)

    def isMappingModified(self):
        site = getSite()
        sm = zope.component.getMultiAdapter(
            (site, self.request), IContentTypeScopeManager)
        return sm.isProfileModified(self.profile_name)


class ContentTypeScopeProfileEditForm(form.EditForm,
        ContentTypeScopeProfileTraverseForm):

    fields = field.Fields(IContentTypeScopeProfileEdit)

    buttons = form.EditForm.buttons.copy()
    handlers = form.EditForm.handlers.copy()

    @button.buttonAndHandler(_('Cancel and Return'), name='cancel')
    def handleCancel(self, action):
        data, errors = self.extractData()
        next_target = '/'.join([self.context.absolute_url(), 
            self.context.__name__, 'view', self.profile_name,])
        self.request.response.redirect(next_target)


class ContentTypeScopeProfileAddForm(form.AddForm):
    
    fields = field.Fields(IContentTypeScopeProfileAdd)

    def update(self):
        self.request['disable_border'] = True
        super(ContentTypeScopeProfileAddForm, self).update()

    def create(self, data):
        result = ContentTypeScopeProfile()
        self._data = data
        return result

    def add(self, obj):
        site = getSite()
        sm = zope.component.getMultiAdapter(
            (site, self.request), IContentTypeScopeManager)
        sm.setEditProfile(self._data['name'], obj)

    def nextURL(self):
        return '/'.join([self.context.absolute_url(), 
            self.context.__name__, 'view', self._data['name'],])


class BaseTokenScopeView(page.SimplePage):

    @property
    def label(self):
        return _(u'Token Scope Information')

    def getTokenKey(self):
        return self.request.get('oauth_token')

    def getScope(self, token_key):
        site = getSite()
        sm = zope.component.getMultiAdapter(
            (site, self.request), IScopeManager)
        return sm.getScope(token_key, None)

    def template(self):
        # This should cause the parent template to render a notification
        return u''

    def getTokenScopeInfo(self):
        # basically check for the existent of the scope.
        token_key = self.getTokenKey()
        if not token_key:
            raise NotFound(self.context, '')

        scope = self.getScope(token_key)
        if not scope:
            raise NotFound(self.context, '')

        return token_key, scope

    def update(self):
        token_key, scope = self.getTokenScopeInfo()


class TokenCTScopeView(BaseTokenScopeView):

    template = ViewPageTemplateFile(path('token_scope_view.pt'))
    missing_metadata = {
        'title': _(u'<Undefined Mapping Type>'),
        'description': u'',
    }

    @property
    def label(self):
        return _(u'Token Scope Information')

    def getScope(self, token_key):
        """
        Override to ensure the correct scope manager is returned.
        """

        site = getSite()
        sm = zope.component.getMultiAdapter(
            (site, self.request), IContentTypeScopeManager)
        # scopes for this manager is a set of mapping_ids.
        return sm.getScope(token_key, None)

    def update(self):
        token_key, mapping_ids = self.getTokenScopeInfo()
        self.request['disable_border'] = True

        site = getSite()
        sm = zope.component.getMultiAdapter(
            (site, self.request), IContentTypeScopeManager)

        # merge the mappings and the profiles.
        all_maps = {}
        writable_maps = {}
        mapping_metadata = []
        mapping_keys = ('portal_type', 'subpaths')
        write_methods = set(('POST', 'PUT', 'DELETE',))
        for mapping_id in mapping_ids:

            # mappings
            mapping = sm.getMapping(mapping_id)
            writable = set(sm.getMappingMethods(mapping_id)
                ).intersection(write_methods)
            for pt, subpaths in mapping.iteritems():
                if not subpaths:
                    continue
                if not pt in all_maps:
                    all_maps[pt] = []
                all_maps[pt].extend(subpaths)

                if writable:
                    if not pt in writable_maps:
                        writable_maps[pt] = []
                    writable_maps[pt].extend(subpaths)

            # profiles
            metadata = sm.getMappingMetadata(mapping_id)
            if not metadata:
                metadata = self.missing_metadata
            mapping_metadata.append(metadata)

        self.mappings = [dict(zip(mapping_keys, (k, sorted(list(set(p))))))
                         for k, p in sorted(all_maps.items())]
        self.writable_mappings = [dict(zip(mapping_keys,
                                           (k, sorted(list(set(p))))))
                                  for k, p in sorted(writable_maps.items())]
        self.hasWritableMappings = len(self.writable_mappings) > 0
        self.profiles = sorted(mapping_metadata,
            lambda a, b: cmp(a['title'], b['title']))



from time import time
import unittest

from zope.interface import Interface
import zope.component
from zope.schema.interfaces import WrongType, WrongContainedType

from zExceptions import Forbidden
from zExceptions import BadRequest

from Products.PloneTestCase import ptc

from pmr2.oauth.interfaces import KeyExistsError
from pmr2.oauth.interfaces import IDefaultScopeManager
from pmr2.oauth.scope import BTreeScopeManager, ContentTypeScopeManager
from pmr2.oauth.scope import ContentTypeScopeProfile

from pmr2.oauth.tests import base


class BTreeScopeManagerTestCase(unittest.TestCase):
    """
    Test the storage and retrieval of client and access key specific
    scopes.
    """

    def setUp(self):
        self.sm = BTreeScopeManager()
        self.client = 'client_key'
        self.access = 'access_key'

    def test_0000_base(self):
        self.assertRaises(KeyError, self.sm.delClientScope, self.client)
        self.assertRaises(KeyError, self.sm.delAccessScope, self.access)

        self.assertRaises(KeyError, self.sm.getClientScope, self.client)
        self.assertRaises(KeyError, self.sm.getAccessScope, self.access)

        self.assertTrue(
            self.sm.getClientScope(self.client, default=None) is None)
        self.assertTrue(
            self.sm.getAccessScope(self.access, default=None) is None)

    def test_0010_set_get(self):
        scope1 = 'http://example.com/scope.1'
        scope2 = 'http://example.com/scope.2'

        self.sm.setClientScope(self.client, scope1)
        self.assertEqual(self.sm.getClientScope(self.client), scope1)

        self.sm.setAccessScope(self.access, scope2)
        self.assertEqual(self.sm.getAccessScope(self.access), scope2)

    def test_0011_set_duplicate(self):
        scope1 = 'http://example.com/scope.1'
        scope2 = 'http://example.com/scope.2'

        self.sm.setClientScope(self.client, scope1)
        self.assertRaises(KeyExistsError, 
            self.sm.setClientScope, self.client, scope1)

        self.sm.setAccessScope(self.access, scope2)
        self.assertRaises(KeyExistsError, 
            self.sm.setAccessScope, self.access, scope2)

    def test_0020_del(self):
        scope1 = 'http://example.com/scope.1'
        scope2 = 'http://example.com/scope.2'

        self.sm.setClientScope(self.client, scope1)
        self.sm.delClientScope(self.client)
        # Can "hide" exception using optional parameter like get/set.
        self.sm.delClientScope(self.client, None)
        self.sm.setClientScope(self.client, scope2)
        self.assertEqual(self.sm.getClientScope(self.client), scope2)

        self.sm.setAccessScope(self.access, scope1)
        # This may be considered dangerous as this does not tie into the
        # access token in any way.  Subclasses may need to handle this
        # to ensure the content owner is aware of scope changes.
        self.sm.delAccessScope(self.access)

        # New scope is updated.
        self.sm.setAccessScope(self.access, scope2)
        self.assertEqual(self.sm.getAccessScope(self.access), scope2)


class CTSMMappingTestCase(unittest.TestCase):
    """
    Testing the profile and management within this scope manager.
    """

    def setUp(self):
        self.sm = ContentTypeScopeManager()
        self.file_mapping = {'File': ['document_view']}
        self.folder_mapping = {'Folder': ['folder_contents']}

    def test_0100_get_mapping(self):
        self.assertRaises(KeyError, self.sm.getMapping, 1)
        self.assertEqual(self.sm.getMapping(1, default='test'), 'test')

    def test_0101_add_get_mapping(self):
        self.assertEqual(self.sm.addMapping('test'), 1)
        self.assertEqual(self.sm.getMapping(1), 'test')

    def test_0200_mapping_name_and_id(self):
        _marker = 2
        self.assertRaises(KeyError, self.sm.getMappingId, 'rawscope')
        self.sm.setMappingNameToId('rawscope', _marker)
        self.assertEqual(self.sm.getMappingId('rawscope'), _marker)
        self.sm.setMappingNameToId('rawscope', 3)
        self.assertEqual(self.sm.getMappingId('rawscope'), 3)
        self.sm.delMappingName('rawscope')
        self.assertRaises(KeyError, self.sm.getMappingId, 'rawscope')

    def test_1000_request_scope_fresh_fail(self):
        self.assertFalse(self.sm.requestScope('key', 'rawscope'))
        self.assertEqual(len(self.sm._scope), 0)

    def test_1001_request_scope_fresh_default(self):
        self.assertTrue(self.sm.requestScope('key', None))
        self.assertEqual(len(self.sm._scope), 1)
        # Can't set this again.
        self.assertRaises(KeyError, self.sm.requestScope, 'key', None)

    def test_1002_request_scope_set_singular(self):
        key = 'request_key'
        scope = 'test_scope'
        mapping_id = self.sm.addMapping(self.file_mapping)
        self.sm.setMappingNameToId(scope, mapping_id)
        self.assertTrue(self.sm.requestScope(key, scope))
        self.assertEqual(len(self.sm._scope), 1)
        mappings = self.sm.getScope(key)
        self.assertEqual(len(mappings), 1)
        self.assertTrue(mapping_id in mappings)
        # Obviously not an access scope.
        self.assertRaises(KeyError, self.sm.getAccessScope, key)
        # Nor a client scope.
        self.assertRaises(KeyError, self.sm.getClientScope, key)

    def test_1003_request_scope_multiple(self):
        file_id = self.sm.addMapping(self.file_mapping)
        folder_id = self.sm.addMapping(self.folder_mapping)
        self.sm.setMappingNameToId('file', file_id)
        self.sm.setMappingNameToId('folder', folder_id)

        key1 = 'request_key1'
        key2 = 'request_key2'
        raw_scope = 'test_scope'
        self.assertFalse(self.sm.requestScope(key1, 'test_scope'))
        # all of them must be valid.
        self.assertFalse(self.sm.requestScope(key1, 
            'http://nohost/plone/scope/file,test_scope'))
        self.assertTrue(self.sm.requestScope(key1, 
            'http://nohost/plone/scope/file,http://nohost/plone/scope/folder'))

        mappings = self.sm.getScope(key1)
        self.assertEqual(len(mappings), 2)
        self.assertTrue(file_id in mappings)
        self.assertTrue(folder_id in mappings)

        self.assertTrue(self.sm.requestScope(key2, 
            'http://nohost/plone/scope/folder'))
        mappings = self.sm.getScope(key2)
        self.assertEqual(len(mappings), 1)
        self.assertTrue(file_id not in mappings)


class CTSMEditingTestCase(unittest.TestCase):
    """
    Testing the profile and management within this scope manager.
    """

    def setUp(self):
        self.sm = ContentTypeScopeManager()
        self.file_profile = ContentTypeScopeProfile()
        self.file_profile.mapping = {'File': ['document_view']}
        self.folder_profile = ContentTypeScopeProfile()
        self.folder_profile.mapping = {'Folder': ['folder_contents']}

    def test_0001_edit(self):
        self.sm.setEditProfile('file', None)
        self.assertRaises(AssertionError, 
            self.sm.setEditProfile, 'file', object())

        self.sm.setEditProfile('file', self.file_profile)
        self.assertEqual(self.sm.getEditProfile('file'), self.file_profile)

    def test_0002_commit_del(self):
        self.sm.setEditProfile('file', self.file_profile)
        self.sm.commitEditProfile('file')
        self.assertEqual(self.sm.getMappingByName('file'),
            self.file_profile.mapping)
        mapping_id = self.sm.getMappingId('file')
        self.assertEqual(self.sm.getMappingMethods(mapping_id),
            ['GET', 'HEAD', 'OPTIONS'])

        self.sm.delMappingName('file')
        self.assertEqual(self.sm.getEditProfile('file'), None)
        self.assertEqual(self.sm.getMappingByName('file', default=None), None)


class CTSMPloneIntegrationTestCase(ptc.PloneTestCase):
    """
    Testing the validation on just the objects with the provided 
    mapping and other Plone integration.
    """

    def afterSetUp(self):
        self.sm = ContentTypeScopeManager()
        self.mapping = {}

    def assertScopeValid(self, accessed, name):
        self.assertTrue(self.sm.validateTargetWithMapping(
            accessed, name, self.mapping))

    def assertScopeInvalid(self, accessed, name):
        self.assertFalse(self.sm.validateTargetWithMapping(
            accessed, name, self.mapping))

    def test_0000_resolve_target(self):
        obj, path = self.sm.resolveTarget(self.folder, 'folder_contents')
        self.assertEqual(obj, 'Folder')
        self.assertEqual(path, 'folder_contents')

    def test_0001_resolve_subtarget(self):
        folder_add = self.folder.restrictedTraverse('+')
        obj, path = self.sm.resolveTarget(folder_add, 'addFolder')
        self.assertEqual(obj, 'Folder')
        self.assertEqual(path, '+/addFolder')

    def test_0002_resolve_nothing(self):
        obj, path = self.sm.resolveTarget(object(), 'addFolder')
        self.assertEqual(obj, None)
        self.assertEqual(path, None)

    def test_0100_root_scope(self):
        self.mapping = {
            'Plone Site': ['folder_contents'],
        }
        self.assertScopeValid(self.portal, 'folder_contents')
        self.assertScopeInvalid(self.portal, 'manage')

    def test_0101_folder_scope(self):
        self.mapping = {
            'Folder': ['folder_contents'],
        }
        self.assertScopeValid(self.folder, 'folder_contents')

    def test_0201_browser_view(self):
        self.mapping = {
            'Folder': ['+/addFile', 'folder_contents'],
        }
        # Adding views.
        folder_add = self.folder.restrictedTraverse('+')
        self.assertScopeValid(folder_add, 'addFile')
        self.assertScopeInvalid(folder_add, 'addFolder')
        self.assertScopeInvalid(self.folder, 'addFile')

        # For whatever reason this happened, but still forbidden by
        # scope restrictions.
        portal_add = self.portal.unrestrictedTraverse('+')
        self.assertScopeInvalid(portal_add, 'addFile')

    def test_0301_asterisk_ending(self):
        self.mapping = {
            'Folder': ['folder*contents', 'test_*'],
            'Plone Site': ['test/test_*', 'test/view*me', 'example/*'],
        }

        self.assertScopeInvalid(self.folder, 'folder_contents')
        self.assertScopeInvalid(self.folder, 'test_view')
        self.assertScopeInvalid(self.folder, 'test_')

        self.assertScopeInvalid(self.portal, 'test_')
        self.assertScopeInvalid(self.portal, 'test/test')
        self.assertScopeInvalid(self.portal, 'test/view_me')
        self.assertScopeValid(self.portal, 'test/test_')
        self.assertScopeValid(self.portal, 'test/test_view')
        self.assertScopeValid(self.portal, 'test/test_page')

        # invalid for now
        self.assertScopeInvalid(self.portal, 'example')
        self.assertScopeValid(self.portal, 'example/')
        self.assertScopeValid(self.portal, 'example/a')


class CTSMValidateTestCase(ptc.PloneTestCase):
    """
    Testing the validation process.
    """

    def afterSetUp(self):
        self.sm = ContentTypeScopeManager()
        self.file_mapping = {'File': ['document_view']}
        self.folder_mapping = {'Folder': ['folder_contents', '+/addFile']}
        file_id = self.sm.addMapping(self.file_mapping)
        folder_id = self.sm.addMapping(self.folder_mapping)
        self.sm.setMappingNameToId('file', file_id)
        self.sm.setMappingNameToId('folder', folder_id)
        self.folder_add = self.folder.restrictedTraverse('+')

        self.all_ids = set([file_id, folder_id])

    def test_0100_request_to_access(self):
        rkey = 'request_key'
        akey = 'access_key'
        self.assertTrue(self.sm.requestScope(rkey, 
            'http://nohost/plone/scope/file,http://nohost/plone/scope/folder'))

        self.sm.setAccessScope(akey, self.sm.getScope(rkey))
        self.assertEqual(self.sm.getAccessScope(akey), self.all_ids)
        # Should there be something to automatically revoke it?
        # Probably?

        request = base.TestRequest()

        self.assertFalse(self.sm.validate(request, '', akey, self.folder,
            self.portal, 'document_view', object()))
        self.assertTrue(self.sm.validate(request, '', akey, self.folder,
            self.portal, 'folder_contents', object()))

        self.assertFalse(self.sm.validate(request, '', akey, self.folder_add,
            self.portal, 'addFolder', object()))
        self.assertTrue(self.sm.validate(request, '', akey, self.folder_add,
            self.portal, 'addFile', object()))

        # TODO test cases where mappings have been purged but the
        # token references to them were not.


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(BTreeScopeManagerTestCase))
    suite.addTest(makeSuite(CTSMMappingTestCase))
    suite.addTest(makeSuite(CTSMEditingTestCase))
    suite.addTest(makeSuite(CTSMPloneIntegrationTestCase))
    suite.addTest(makeSuite(CTSMValidateTestCase))
    return suite



import zope.interface
import zope.schema

from pmr2.oauth import MessageFactory as _


class KeyExistsError(KeyError):
    __doc__ = "key exists error"


class BaseInvalidError(KeyError):
    __doc__ = "base invalid error."


class TokenInvalidError(BaseInvalidError):
    __doc__ = "invalid token."


class ConsumerInvalidError(BaseInvalidError):
    __doc__ = "invalid consumer."


class RequestInvalidError(BaseInvalidError):
    __doc__ = "invalid request."


class BaseValueError(ValueError):
    __doc__ = "basic value error"


class NotRequestTokenError(TokenInvalidError):
    __doc__ = "Not request token"


class NotAccessTokenError(TokenInvalidError):
    __doc__ = "Not access token"


class ExpiredTokenError(TokenInvalidError):
    __doc__ = "Expired token"


class CallbackValueError(BaseValueError):
    __doc__ = "callback value error"


class NonceValueError(BaseValueError):
    __doc__ = "nonce value error"


class InvalidScopeError(BaseValueError):
    __doc__ = "invalid scope."


class IOAuthAdapter(zope.interface.Interface):
    """
    Interface for the OAuth adapter.
    """

    def __call__():
        """
        Return a boolean value to determine whether access was granted.
        """


class IOAuthPlugin(zope.interface.Interface):
    """\
    The OAuth plugin.
    """

    def extractOAuthCredentials(request):
        """\
        Extract the OAuth credentials from the request, for processing
        by Plone PAS.
        """


class IConsumer(zope.interface.Interface):
    """\
    An OAuth consumer.
    """

    key = zope.schema.ASCIILine(
        title=_(u'Consumer Key'),
        description=_(u'The key that identifies this consumer.  This usually '
                     'is the domain name of the consumer'),
        required=True,
    )

    secret = zope.schema.ASCIILine(
        title=_(u'Consumer Secret'),
        description=_(u'Consumer secret'),
        required=True,
    )

    title = zope.schema.TextLine(
        title=_(u'Consumer Title'),
        description=_(u'Short description of what or who this consumer is.'),
        required=False,
    )

    domain = zope.schema.TextLine(
        title=_(u'Domain name'),
        description=_(u'The domain that is permitted to receive redirection '
                       'requests.'),
        required=False,
    )

    def validate():
        """
        Self validation.
        """


class IConsumerManager(zope.interface.Interface):
    """\
    Consumer utility
    """

    def add(consumer):
        """\
        Add a consumer.
        """

    def check(consumer):
        """\
        Check for validity of input consumer.
        """

    def get(consumer_key, default=None):
        """\
        Return consumer, identified by consumer_key.
        """

    def getAllKeys():
        """\
        Return all consumer keys tracked by this consumer.
        """

    def getValidated(consumer_key, default=None):
        """\
        Return consumer only if it is a validated one.

        This will be used when possible to allow further checks by 
        alternative implementations.
        """

    def remove(consumer):
        """\
        Remove consumer.
        """


class IScopeManager(zope.interface.Interface):
    """\
    Scope Manager

    A manager that simplifies the handling of scopes, which place limits
    on what an authenticated token can access.
    """

    def setScope(key, scope):
        """
        Set a scope identified by key.
        """

    def getScope(key, scope):
        """
        Get a scope identified by key.
        """

    def popScope(key, scope):
        """
        Pop out a scope identified by key
        """

    def setClientScope(client_key, scope):
        """
        Set the scope provided by client, referenced by client_key.
        """

    def setAccessScope(access_key, scope):
        """
        Set the scope provided by access, referenced by access_key.
        """

    def getClientScope(client_key, default):
        """
        Get the scope for the provided client_key.
        """

    def getAccessScope(access_key, default):
        """
        Get the scope for the provided access_key.
        """

    def delClientScope(client_key):
        """
        Delete the scope for the provided client_key.
        """

    def delAccessScope(access_key):
        """
        Delete the scope for the provided access_key.
        """

    def requestScope(request_key, rawscope):
        """
        Request a scope for the temporary credentials identified by the
        ``request_key``.

        request_key
            the generated request key.
        rawscope
            the raw scope string sent by the client.

        Return True if the rawscope is successfully stored as a scope
        with the request_key, False otherwise.
        
        The actual scope object can be retrieved by calling
        `self.getScope(request_key)` if this was successful.
        """

    def validate(request, client_key, access_key,
            accessed, container, name, value):
        """
        Validate the scope against the given context with the given
        client and owner.

        request
            the request object.

        client_key
            the client (consumer) key.

        access_key
            the access key identifying a given token granted by a
            resource owner.

        accessed
            the immediate object accessed by the client before the
            value

        container
            the real container of the value

        name
            the name used by the client to access the value.

        value
            the value accessed by the client. 

        The latter four fields are normally called by keywords.
        """


class IDefaultScopeManager(IScopeManager):
    """
    Marker interface for the default scope manager.
    """


class IContentTypeScopeManager(IScopeManager):
    """
    A scope manager based on content types.

    A scope manager validates the requested object and the name with a
    content type profile specific to the client and/or resource access
    key if available, or the default profile if not.
    """

    default_mapping_id = zope.schema.Int(
        title=_(u'Default Mapping ID'),
        required=True,
        default=0,
    )

    def resolveProfile(client_key, access_key):
        """
        Reolve the provided client_key and access_key into a validation
        profile.
        """

    def resolveTarget(accessed, name):
        """
        Resolve the accessed item and name into a target for the next
        method.
        """

    def validateTargetWithProfile(accessed_typeid, subpath, profile):
        """
        The scope value will resolve into a profile which is used to
        validate against the provided parameters.
        """


class IContentTypeScopeProfile(zope.interface.Interface):
    """
    Interface for the scope profile and editor.
    """

    title = zope.schema.TextLine(
        title=_(u'Title'),
        description=_(
            u'Brief description about this scope profile.'),
        required=True,
    )

    description = zope.schema.Text(
        title=_(u'Description'),
        description=_(
            u'Detailed description of the rights granted by this scope.'),
        required=False,
    )

    methods = zope.schema.ASCIILine(
        title=_(u'Permitted HTTP Methods'),
        description=_(
            u'Whitespace delimited list of permitted HTTP methods for the '
            'subpaths below.'),
        required=True,
        default='GET HEAD OPTIONS',
    )

    mapping = zope.schema.Dict(
        title=_(u'Mapping'),
        description=_(u'A mapping for each of the following portal types to '
                     'a list of permitted subpaths.'),
        key_type=zope.schema.ASCIILine(
            title=_(u'Portal Type')
        ),
        value_type=zope.schema.List(
            title=_(u'Permitted subpaths'),
            value_type=zope.schema.ASCIILine(title=_(u'Subpath')),
            required=False,
        ),
    )


class IToken(zope.interface.Interface):
    """\
    An OAuth token.
    """

    key = zope.schema.ASCIILine(
        title=_(u'Key'),
        description=_(u'Consumer key'),
        required=True,
    )

    secret = zope.schema.ASCIILine(
        title=_(u'Secret'),
        description=_(u'Consumer secret'),
        required=True,
    )

    callback = zope.schema.ASCIILine(
        title=_(u'Callback'),
        required=False,
    )

    verifier = zope.schema.ASCIILine(
        title=_(u'Verifier'),
        required=True,
    )

    # other requirements

    access = zope.schema.Bool(
        title=_(u'Access Permitted'),
        description=_(u'Determines if this can be used to access content.'),
        default=False,
        required=True,
    )

    user = zope.schema.ASCIILine(
        title=_(u'User ID'),
        description=_(u'The user id associated with this token.'),
        required=False,
        default=None,
    )

    consumer_key = zope.schema.ASCIILine(
        title=_(u'Consumer Key'),
        description=_(u'The consumer key associated with this token'),
        required=False,
        default=None,
    )

    timestamp = zope.schema.Int(
        title=_(u'Timestamp'),
        description=_(u'Creation timestamp of this token'),
    )

    expiry = zope.schema.Int(
        title=_(u'Expiry'),
        description=_(u'Expiry timestamp for this token'),
    )

    def validate():
        """
        Self validation.
        """


class ITokenManager(zope.interface.Interface):
    """\
    Token manager utility
    """

    def add(token):
        """\
        Add a token.
        """

    def generateRequestToken(consumer, request):
        """\
        Generate a request token, using consumer and request.
        """

    def generateAccessToken(consumer, request):
        """\
        Generate an access token.
        """

    def claimRequestToken(token, user):
        """\
        Token claimed by user.
        """

    def get(token, default=None):
        """\
        Get a token by token.

        Input could be a token, or a key.  Returns the same token 
        identified by the key of the input token or the input key.
        """

    def getRequestToken(token, default=False):
        """\
        Return request token identified by token.

        Raises NotRequestTokenError when token is not an access token.
        Raises InvalidTokenError if internal consistency (invariants)
        are violated.
        If token is not found and default value is false, 
        InvalidTokenError should be raised also.
        """

    def getAccessToken(token, default=False):
        """\
        Return access token identified by token.

        Raises NotAccessTokenError when token is not an access token.
        Raises InvalidTokenError if internal consistency (invariants)
        are violated.
        If token is not found and default value is false, 
        InvalidTokenError should be raised also.
        """

    def getTokensForUser(user):
        """\
        Return a list of token keys for a user.
        """

    def remove(token):
        """\
        Remove token.
        """


# Other management interfaces

class ICallbackManager(zope.interface.Interface):
    """
    Callback manager.

    Used to verify the validity of callback URIs.
    """

    def validate(consumer, token):
        """
        Check that the callbacks are valid against both the consumer and
        the token.  A more thorough implementation should allow multiple
        hosts for consumers, matching against the tokens issued, instead
        of just relying on the helper attribute provided by consumer.

        token
            The token to validate against.
        consumer
            The consumer to validate against.
        """


class INonceManager(zope.interface.Interface):
    """\
    Nonce manager.

    If nonce must be checked specifically, implement this manager.
    """

    def check(timestamp, nonce):
        """\
        Check that this nonce can be used.
        """


class _IDynamicSchemaInterface(zope.interface.Interface):
    """
    Placeholder
    """



from logging import getLogger
from zope.app.component.hooks import getSite

from Products.CMFCore.utils import getToolByName


def createPlugin(site, id_):
    logger = getLogger('pmr2.oauth')
    acl = getToolByName(site, "acl_users")
    acl.manage_addProduct["pmr2.oauth"].addOAuthPlugin(
            id=id_, title='OAuth authentication plugin')


def activatePlugin(site, id_):
    logger = getLogger('pmr2.oauth')
    acl = getToolByName(site, "acl_users")
    plugin = getattr(acl, id_)
    interfaces = plugin.listInterfaces()

    activate = []

    for info in acl.plugins.listPluginTypeInfo():
        interface = info["interface"]
        interface_name = info["id"]
        if plugin.testImplements(interface):
            activate.append(interface_name)
            logger.info("Activating interface %s for plugin %s" % 
                    (interface_name, info["title"]))

    plugin.manage_activateInterfaces(activate)


def importVarious(context):
    # Only run step if a flag file is present (e.g. not an extension profile)
    if context.readDataFile('oauth-pas.txt') is None:
        return

    id_ = 'oauth'
    site = context.getSite()

    acl = getToolByName(site, "acl_users")
    installed = acl.objectIds()
    if id_ not in installed:
        createPlugin(site, id_)
        activatePlugin(site, id_)


def migrate_v0_2_to_v_0_4(context):
    logger = getLogger('pmr2.oauth')
    logger.info('Migrating pmr2.oauth to v0.4.')
    site = getSite()
    scope_upgrade_v0_4(site)
    context.runAllImportStepsFromProfile('profile-pmr2.oauth:default')

def scope_upgrade_v0_4(site):
    import zope.component
    from zope.annotation import IAnnotations
    from pmr2.oauth.interfaces import ITokenManager, IConsumerManager

    logger = getLogger('pmr2.oauth')
    ants = IAnnotations(site)
    if 'pmr2.oauth.scope.DefaultScopeManager' in ants:
        logger.info('Purging the removed DefaultScopeManager.')
        del ants['pmr2.oauth.scope.DefaultScopeManager']

    # The following needs to be done because of the uselessness of a
    # token without a scope, not to mention the impossibility of
    # migrating the scope definitions from previous scope manager.
    logger.info('Purging and reinitializing the built-in token manager.')
    tm = zope.component.getMultiAdapter((site, None), ITokenManager)
    tm.__init__()



import zope.component
import zope.interface

from Products.CMFCore.utils import getToolByName

from pmr2.z3cform import page

from pmr2.oauth import MessageFactory as _
from pmr2.oauth.browser.template import path, ViewPageTemplateFile


class PMR2OAuthPage(page.SimplePage):

    template = ViewPageTemplateFile(path('pmr2-oauth.pt'))
    label = _(u'pmr2.oauth views')

    @property
    def portal_url(self):
        portal_url = getToolByName(self.context, 'portal_url', None)
        if portal_url:
            portal = portal_url.getPortalObject()
            return portal.absolute_url()

    def isManager(self):
        mt = getToolByName(self.context, 'portal_membership')
        user = mt.getAuthenticatedMember()
        return user.has_role('Manager')

    def update(self):
        self.request['disable_border'] = True



import zope.component
from zope.annotation import IAnnotations

from BTrees.OOBTree import OOBTree

from Products.PloneTestCase import ptc

from pmr2.oauth.interfaces import ITokenManager

from pmr2.oauth.tests import base


class MigrationV04TestCase(ptc.PloneTestCase):
    """
    Test case for migration from v0.2 to v0.4
    """

    def afterSetUp(self):
        # old annotation
        ants = IAnnotations(self.portal)
        sm = type('DefaultScopeManager', (object, ),
            {'__module__': 'pmr2.oauth.scope'})
        ants['pmr2.oauth.scope.DefaultScopeManager'] = sm()
        # Might be better to mock this up properly.
        tm = zope.component.getMultiAdapter((self.portal, None), ITokenManager)
        tm._tokens = OOBTree()
        tm._user_token_map = OOBTree()
        tm._tokens.insert('test1', object())
        tm._tokens.insert('test2', object())
        tm._tokens.insert('test3', object())

    def test_0000_migration(self):
        from pmr2.oauth.setuphandlers import scope_upgrade_v0_4
        tm = zope.component.getMultiAdapter((self.portal, None), ITokenManager)
        self.assertEqual(len(tm._tokens), 3)
        scope_upgrade_v0_4(self.portal)
        # 1 for the dummy token.
        self.assertEqual(len(tm._tokens), 1)
        ants = IAnnotations(self.portal)
        self.assertFalse('pmr2.oauth.scope.DefaultScopeManager' in ants)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(MigrationV04TestCase))
    return suite



import zope.interface
import zope.schema

from pmr2.oauth import MessageFactory as _
from pmr2.oauth.schema import SchemaMethodObject
from pmr2.oauth.utility import buildContentTypeScopeProfileInterface

from pmr2.oauth.interfaces import IContentTypeScopeProfile


class IContentTypeScopeProfileAdd(zope.interface.Interface):
    """
    Fields for the scope profile add form.
    """

    name = zope.schema.ASCIILine(
        title=_(u'Profile Name'),
        description=_(u'The name for the scope profile.'),
        required=True,
    )


class IContentTypeScopeProfileItem(zope.interface.Interface):
    """
    Fields for the scope profile edit form.  This is for the individual
    items.
    """

    name = zope.schema.ASCII(
        title=_(u'Permitted Views'),
        description=_(u'List of views identified by their name that are '
                       'permitted for this content type.'),
        required=False,
    )


class IContentTypeScopeProfileEdit(zope.interface.Interface):
    """
    Interface for editing a scope profile.
    """

    title = zope.schema.TextLine(
        title=_(u'Title'),
        description=_(
            u'Brief description about this scope profile.'),
        required=True,
    )

    description = zope.schema.Text(
        title=_(u'Description'),
        description=_(u'A description of the rights granted by this scope.'),
        required=False,
    )

    methods = zope.schema.ASCIILine(
        title=_(u'Permitted HTTP Methods'),
        description=_(
            u'Whitespace delimited list of permitted HTTP methods for the '
            'subpaths below.'),
        required=True,
        default='GET HEAD OPTIONS',
    )

    mapping = SchemaMethodObject(
        title=_(u'Mapping'),
        description=_(u'A mapping for each of the following portal types to '
                       'a list of permitted subpaths.'),
        schema=buildContentTypeScopeProfileInterface,
        required=False,
    )



import traceback
import logging

import zope.component
from zope.app.component.hooks import getSite

from AccessControl.SecurityInfo import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
from Products.PluggableAuthService.utils import classImplements
from Products.PluggableAuthService.interfaces.plugins \
                import IAuthenticationPlugin, IExtractionPlugin

from zExceptions import Forbidden
from zExceptions import BadRequest

from pmr2.oauth.interfaces import IOAuthPlugin, IOAuthAdapter, IScopeManager


manage_addOAuthPlugin = PageTemplateFile("../www/oauthAdd", globals(), 
                __name__="manage_addOAuthPlugin")

logger = logging.getLogger("PluggableAuthService")

def addOAuthPlugin(self, id, title='', REQUEST=None):
    """Add an OAuth plugin to a Pluggable Authentication Service.
    """
    p = OAuthPlugin(id, title)
    self._setObject(p.getId(), p)

    if REQUEST is not None:
        REQUEST["RESPONSE"].redirect("%s/manage_workspace"
                "?manage_tabs_message=OAuth+plugin+added." %
                self.absolute_url())

class OAuthPlugin(BasePlugin):
    """OAuth authentication plugin.
    """

    # to implement IExtractionPlugin, IAuthenticationPlugin
    meta_type = "OAuth plugin"
    security = ClassSecurityInfo()
    zope.interface.implements(IOAuthPlugin)

    def __init__(self, id, title=None):
        self._setId(id)
        self.title = title

    # IExtractionPlugin implementation
    def extractCredentials(self, request):
        """\
        This method performs the PAS credential extraction.

        Consumer sends in credentials, which gets extracted.

        If the credential had been authenticated, return the login id,
        otherwise empty mapping.
        """

        if not (request._auth and request._auth.startswith('OAuth ')):
            # Skip all not OAuth related.
            return {}

        site = getSite()
        verifier = zope.component.getMultiAdapter(
            (site, request), IOAuthAdapter)

        try:
            result = verifier()
        except ValueError:
            raise BadRequest('bad oauth request')

        if result is None:
            # Valid request, but yielded no access key that will allow
            # this plugin to return a credential.
            return {}

        if result is False:
            raise Forbidden('authorization failed')

        # Please see _validateScope
        scope = self._validateScope(site, request,
            verifier.client_key, verifier.access_key)
        if scope is False:
            raise Forbidden('invalid scope')

        if scope is None:
            # No scope manager?
            return {}

        mappings = {}
        token = verifier.tokenManager.getAccessToken(verifier.access_key)
        mappings['userid'] = token.user
        return mappings

    def _validateScope(self, site, request, client_key, access_key):
        # This should really be done outside of here by a customized
        # SecurityManager/Policy, which PAS will invoke some time after
        # this plugin is called.  However I have no time to figure out 
        # how or where to start, so a short-circuit process is done here 
        # for now.
        
        pas = self._getPAS()
        accessed, container, name, value = pas._getObjectContext(
            request.PUBLISHED, request)

        scopeManager = zope.component.queryMultiAdapter(
            (site, request), IScopeManager)
        if not scopeManager:
            # This normally shouldn't happen...
            return
        
        scopeValidity = scopeManager.validate(request, client_key, access_key, 
            accessed=accessed,
            container=container,
            name=name,
            value=value,
        )

        return scopeValidity

    # IAuthenticationPlugin implementation
    def authenticateCredentials(self, credentials):
        """\
        Authenticate the generated credentials by above.
        """

        if not credentials.get('extractor', None) == 'oauth':
            return None

        userid = credentials['userid']
        pas = self._getPAS()
        info = pas._verifyUser(pas.plugins, user_id=userid)
        if info is None:
            return None  # should we raise Forbidden instead?

        return (info['id'], info['login'])

classImplements(OAuthPlugin,
                IExtractionPlugin,
                IAuthenticationPlugin)



import re

from persistent import Persistent
from BTrees.OOBTree import OOBTree
from BTrees.IOBTree import IOBTree
from BTrees.OIBTree import OIBTree

from zope.app.container.contained import Contained
from zope.annotation.interfaces import IAttributeAnnotatable
import zope.interface
from zope.schema import fieldproperty

from Acquisition import aq_parent, aq_inner
from Products.CMFCore.utils import getToolByName

from pmr2.oauth.interfaces import KeyExistsError
from pmr2.oauth.interfaces import IScopeManager, IDefaultScopeManager
from pmr2.oauth.interfaces import IContentTypeScopeManager
from pmr2.oauth.interfaces import IContentTypeScopeProfile
from pmr2.oauth.factory import factory

_marker = object()


class BaseScopeManager(object):
    """
    Base scope manager.

    The base scope manager, does nothing on its own, can be used as a
    boilerplate for other scope managers.
    """

    zope.interface.implements(IScopeManager)
    
    def __init__(self):
        pass

    def setScope(self, key, scope):
        raise NotImplementedError()

    def getScope(self, key, default=_marker):
        raise NotImplementedError()

    def popScope(self, key, default=_marker):
        raise NotImplementedError()

    def setClientScope(self, client_key, scope):
        raise NotImplementedError()

    def setAccessScope(self, access_key, scope):
        raise NotImplementedError()

    def getClientScope(self, client_key, default):
        raise NotImplementedError()

    def getAccessScope(self, access_key, default):
        raise NotImplementedError()

    def delClientScope(self, client_key):
        raise NotImplementedError()

    def delAccessScope(self, access_key):
        raise NotImplementedError()

    def validate(self, request, client_key, access_key,
            accessed, container, name, value):
        raise NotImplementedError()

    def requestScope(self, request_key, raw_scope):
        raise NotImplementedError()


class BTreeScopeManager(Persistent, Contained, BaseScopeManager):
    """
    Basic BTree based client/access scope manager.

    Provides mapping of client and access keys to a scope, but does not
    provide any validation capabilities.
    """

    zope.component.adapts(IAttributeAnnotatable, zope.interface.Interface)

    client_prefix = 'client.'
    access_prefix = 'access.'

    def __init__(self):
        self._scope = OOBTree()

    def setScope(self, key, scope):
        if self._scope.get(key, _marker) != _marker:
            raise KeyExistsError()
        self._scope[key] = scope

    def getScope(self, key, default=_marker):
        result = self._scope.get(key, default)
        if result == _marker:
            raise KeyError()
        return result

    def popScope(self, key, default=_marker):
        result = self._scope.pop(key, default)
        return result

    def setClientScope(self, client_key, scope):
        key = self.client_prefix + client_key
        self.setScope(key, scope)

    def setAccessScope(self, access_key, scope):
        key = self.access_prefix + access_key
        self.setScope(key, scope)

    def getClientScope(self, client_key, default=_marker):
        key = self.client_prefix + client_key
        return self.getScope(key, default)

    def getAccessScope(self, access_key, default=_marker):
        key = self.access_prefix + access_key
        return self.getScope(key, default)

    def delClientScope(self, client_key, default=_marker):
        key = self.client_prefix + client_key
        result = self.popScope(key, default)
        if result == _marker:
            raise KeyError()

    def delAccessScope(self, access_key, default=_marker):
        key = self.access_prefix + access_key
        result = self.popScope(key, default)
        if result == _marker:
            raise KeyError()

    def requestScope(self, request_key, raw_scope):
        """
        Requesting scope for this key.
        """

        # No reason or means to refuse this request as this doesn't do
        # any kind of management.
        self.setScope(request_key, raw_scope)
        return True


class ContentTypeScopeManager(BTreeScopeManager):
    """
    A scope manager based on content types.

    This scope manager validates the request using the content type of
    the accessed object and the subpath of the request against a content
    type mapping.  The content type mapping to be used will be one of
    specified by the resource access key, the client key or default, and
    is resolved in this order.

    One more restriction imposed by this scope manager: mappings are
    enforced absolutely for access keys.  This allows clients to request
    new default scopes for themselves at will and/or have site-wide
    default scope changes without compromising the scopes already
    granted by the resource owner referenced by the access key.

    This however does not address the case where additional global
    restrictions that may be placed by the site owner as the focus is
    ultimately on the access keys.  Workaround is to revoke those keys
    and have the content owners issue new ones regardless of changes.

    Pruning of unused scope is not implemented.
    """

    zope.interface.implements(IContentTypeScopeManager)

    default_mapping_id = fieldproperty.FieldProperty(
        IContentTypeScopeManager['default_mapping_id'])

    def __init__(self):
        super(ContentTypeScopeManager, self).__init__()
        self._mappings = IOBTree()

        # Methods permitted to access this mapping with.  Originally
        # I wanted to provide alternative sets of mapping on a per
        # mapping_id basis, however this proved to be complex and
        # complicated due to extra relationships involved.
        self._methods = IOBTree()

        # For metadata related to the above.
        self._mappings_metadata = IOBTree()

        # To ease the usage of scopes, the mappings are referenced by
        # names and are called profiles which add a few useful fields to
        # allow slightly easier usage.  This separates the name from the
        # already active tokens such that once a token is instantiated
        # with a scope, the mapping is stuck until the token is revoked.
        self._named_mappings = OIBTree()  # name to id.

        # To not overburden the named mappings with work-in-progress
        # profiles, instantiate one here also.
        self._edit_mappings = OOBTree()

        self.default_mapping_id = self.addMapping({})

    # Main mapping related management methods.

    def addMapping(self, mapping, methods='GET HEAD OPTIONS', metadata=None):
        key = 0  # default?
        if len(self._mappings) > 0:
            # Can calculate the next key.
            key = self._mappings.maxKey() + 1
        self._mappings[key] = mapping
        self._methods[key] = methods.split()
        if metadata is not None:
            self._mappings_metadata[key] = metadata
        return key

    def getMapping(self, mapping_id, default=_marker):
        result = self._mappings.get(mapping_id, default)
        if result is _marker:
            raise KeyError()
        return result

    def getMappingMetadata(self, mapping_id, default=None):
        result = self._mappings_metadata.get(mapping_id, default)
        return result

    def getMappingId(self, name):
        # Returned ID could potentially not exist, what do?
        return self._named_mappings[name]

    def getMappingMethods(self, mapping_id, default=_marker):
        result = self._methods.get(mapping_id, default)
        if result is _marker:
            raise KeyError()
        return result

    def checkMethodPermission(self, mapping_id, method):
        methods = self.getMappingMethods(mapping_id, ())
        return method in methods

    def setMappingNameToId(self, name, mapping_id):
        self._named_mappings[name] = mapping_id

    def delMappingName(self, name):
        saved = self._named_mappings.pop(name, None)
        edits = self._edit_mappings.pop(name, None)
        return (saved, edits)

    def getMappingByName(self, name, default=_marker):
        try:
            mapping_id = self.getMappingId(name)
            mapping = self.getMapping(mapping_id)
        except KeyError:
            if default == _marker:
                raise
            mapping = default
        return mapping

    def getMappingNames(self):
        return self._named_mappings.keys()

    # Temporary/edited mapping profiles

    def getEditProfile(self, name, default=None):
        return self._edit_mappings.get(name, default)

    def setEditProfile(self, name, value):
        assert IContentTypeScopeProfile.providedBy(value) or value is None
        self._edit_mappings[name] = value

    def commitEditProfile(self, name):
        profile = self.getEditProfile(name)
        if not (IContentTypeScopeProfile.providedBy(profile)):
            raise KeyError('edit profile does not exist')
        new_mapping = profile.mapping
        methods = profile.methods
        metadata = {
            'title': profile.title,
            'description': profile.description,
            # Should really not duplicate this there but this is easy
            # shortcut to take for now.
            'methods': methods,
        }
        new_id = self.addMapping(new_mapping, methods=methods,
            metadata=metadata)
        self.setMappingNameToId(name, new_id)

    def getEditProfileNames(self):
        return self._edit_mappings.keys()

    def isProfileModified(self, name):
        # TODO I would like some way to compare the two profiles in a
        # sane way but only using active types and types that have
        # stuff assigned.  So for now just use this naive method.
        profile = self.getEditProfile(name)
        try:
            mapping_id = self.getMappingId(name)
            mapping = self.getMapping(mapping_id)
            metadata = self.getMappingMetadata(mapping_id, {})
        except KeyError:
            # If profile exists, no associated ID, definitely modified.
            return True

        return not (profile.mapping == mapping and 
            profile.title == metadata.get('title') and
            profile.description == metadata.get('description') and
            profile.methods == metadata.get('methods')
        )

    # Scope handling.

    def requestScope(self, request_key, raw_scope):
        """
        This manager references scope by ids internally.  Resolve the
        raw scope id by the client into the mapping ids.
        """

        raw_scopes = raw_scope and raw_scope.split(',') or []
        result = set()
        for rs in raw_scopes:
            # Ignoring the current site URI and just capture the final
            # fragment.
            name = rs.split('/')[-1]
            try:
                mapping_id = self.getMappingId(name)
                # This verifies the existence of the mapping with id.
                mapping = self.getMapping(mapping_id)
            except KeyError:
                # Failed to fulfill the requested scope.
                return False
            result.add(mapping_id)

        if not result:
            result.add(self.default_mapping_id)

        self.setScope(request_key, result)
        return True

    def validate(self, request, client_key, access_key,
            accessed, container, name, value):
        """
        See IScopeManager.
        """

        mappings = self.resolveMapping(client_key, access_key)
        # multiple rights were requested, check through all of them.
        for mapping_id in mappings:
            mapping = self.getMapping(mapping_id, default={})
            result = self.validateTargetWithMapping(accessed, name, mapping)
            method_allowed = self.checkMethodPermission(mapping_id,
                request.method)
            if result and method_allowed:
                return True

        # no matching mappings.
        return False

    def resolveMapping(self, client_key, access_key):
        """
        See IDefaultScopeManager.
        """

        # As all mappings are referenced byh access keys.
        return self.getAccessScope(access_key, None)

    def resolveTarget(self, accessed, name):
        """
        Accessed target resolution.

        Find the type of the container object of the accessed object by
        traversing upwards, and gather the path to resolve into the 
        content type id.  Return both these values.
        """

        # use getSite() instead of container?
        pt_tool = getToolByName(accessed, 'portal_types', None)
        if pt_tool is None:
            return None, None

        context = aq_inner(accessed)
        typeinfo = None
        subpath = [name]

        while context is not None:
            typeinfo = pt_tool.getTypeInfo(context)
            if typeinfo:
                subpath.reverse()
                return typeinfo.id, '/'.join(subpath)
            # It should have a name...
            subpath.append(context.__name__)
            context = aq_parent(context)

        return None, None

    def validateTargetWithMapping(self, accessed, name, mapping):
        atype, subpath = self.resolveTarget(accessed, name)
        return self.validateTypeSubpathMapping(atype, subpath, mapping)

    def validateTypeSubpathMapping(self, accessed_type, subpath, mapping):
        # A simple lookup method.
        valid_scopes = mapping.get(accessed_type, {})
        if not valid_scopes:
            return False

        for vs in valid_scopes:
            # XXX ignores second last asterisk, preventing validation
            # against items that have an asterisk in its name for 
            # whatever reason...
            if vs.endswith('*') and '/' in vs:
                match = subpath.startswith(vs[:vs.rindex('*')])
            else:
                match = subpath == vs
            if match:
                return True
        return False

ContentTypeScopeManagerFactory = factory(ContentTypeScopeManager)


class ContentTypeScopeProfile(Persistent):
    """
    The one for editing purpose.  Allows definition of names and fields
    related to the user side creation and usage of mappings.
    """

    zope.interface.implements(IContentTypeScopeProfile)

    title = fieldproperty.FieldProperty(
        IContentTypeScopeProfile['title'])
    description = fieldproperty.FieldProperty(
        IContentTypeScopeProfile['description'])
    methods = fieldproperty.FieldProperty(IContentTypeScopeProfile['methods'])
    mapping = fieldproperty.FieldProperty(IContentTypeScopeProfile['mapping'])



import zope.i18nmessageid
from AccessControl.Permissions import manage_users as ManageUsers
from Products.PluggableAuthService.PluggableAuthService import \
    registerMultiPlugin

MessageFactory = zope.i18nmessageid.MessageFactory('pmr2.oauth')

from plugins import oauth


registerMultiPlugin(oauth.OAuthPlugin.meta_type)

def initialize(context):
    # XXX should validate whether we have SSL installed.

    context.registerClass(oauth.OAuthPlugin,
        permission=ManageUsers,
        constructors=(
            oauth.manage_addOAuthPlugin,
            oauth.addOAuthPlugin,
        ),
        visibility=None,
        icon="icon/oauth.png"
    )



import doctest
import unittest

from zope.component import testing
from Testing import ZopeTestCase as ztc

from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import PloneSite
from Products.PloneTestCase.layer import onsetup

from pmr2.z3cform.tests.base import DocTestCase

from pmr2.oauth.tests import base


def test_suite():
    return unittest.TestSuite([

        ztc.ZopeDocFileSuite(
            'README.rst', package='pmr2.oauth',
            test_class=DocTestCase,
            optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
        ),

    ])



import urllib

import zope.component
import zope.interface
from zope.app.component.hooks import getSite
from zope.publisher.browser import BrowserPage
from zope.publisher.interfaces import NotFound

from zExceptions import BadRequest
from zExceptions import Forbidden
from zExceptions import Unauthorized

from z3c.form import button

from Products.CMFCore.utils import getToolByName

from pmr2.z3cform import form

from pmr2.oauth import MessageFactory as _
from pmr2.oauth.interfaces import *
from pmr2.oauth.browser.template import ViewPageTemplateFile
from pmr2.oauth.browser.template import path

_marker = object()


class BaseTokenPage(BrowserPage):

    # token to return
    token = None

    def _verifyToken(self, oauth):
        # Return the correct OAuth method for the respective token 
        # request page.
        raise NotImplementedError()

    def getOAuth1(self):
        if not hasattr(self.request, '_pmr2_oauth1_'):
            site = getSite()
            oauthAdapter = zope.component.getMultiAdapter((site, self.request),
                IOAuthAdapter)
            try:
                result, oauth1 = self._verifyToken(oauthAdapter)
            except ValueError:
                raise BadRequest()
            if not result:
                raise Forbidden()
            #self.request._pmr2_oauth1_ = oauth1
            return oauth1
        else:
            # Prepared by the run at the plugin.
            return self.request._pmr2_oauth1_

    def update(self):
        # Considering modifying to call something like create and store
        # token and store scope.
        raise NotImplementedError()

    def render(self):
        token = self.token
        data = {
            'oauth_token': token.key,
            'oauth_token_secret': token.secret,
        }
        if token.callback is not None:
            data['oauth_callback_confirmed'] = 'true'
        return urllib.urlencode(data)

    def __call__(self):
        self.update()
        return self.render()


class RequestTokenPage(BaseTokenPage):

    def _verifyToken(self, oauth):
        # See parent class
        return oauth.verify_request_token_request()

    def update(self):
        oauth1 = self.getOAuth1()

        # NOTE Currently it is impossible to disable callback validation
        # in oauthlib, so verify that callback is really provided.
        if not oauth1.callback_uri:
            raise BadRequest()

        # This is an 8-bit protocol, so we cast the oauthlib request
        # parameters into something we would expect from the http spec.
        # Yes, I am aware something about unicode literals in Python 3,
        # but this transition period is just a giant pita.
        # If these kind of casting dies in a fire I don't really care
        # because these should all be ascii for simplicity.
        consumer_key = str(oauth1.client_key)
        callback = str(oauth1.callback_uri)

        # create request token
        tm = zope.component.getMultiAdapter((self.context, self.request),
            ITokenManager)
        self.token = tm.generateRequestToken(consumer_key, callback)

        # store the scope.
        scope = self.request.get('scope', None)
        key = self.token.key
        sm = zope.component.getMultiAdapter((self.context, self.request),
            IScopeManager)
        if not sm.requestScope(key, scope):
            raise Forbidden()


class GetAccessTokenPage(BaseTokenPage):

    def _verifyToken(self, oauth):
        # See parent class
        return oauth.verify_access_token_request()

    def update(self):
        oauth1 = self.getOAuth1()

        consumer_key = str(oauth1.client_key)
        token_key = str(oauth1.resource_owner_key)
        verifier = str(oauth1.verifier)

        tm = zope.component.getMultiAdapter((self.context, self.request),
            ITokenManager)
        self.token = tm.generateAccessToken(consumer_key, token_key)

        # move the scope stored into the access token.
        sm = zope.component.getMultiAdapter((self.context, self.request),
            IScopeManager)

        scope = sm.popScope(token_key, _marker)
        if scope == _marker:
            # could not find the scope that was stored.
            raise Forbidden()

        key = self.token.key
        sm.setAccessScope(key, scope)


class AuthorizeTokenForm(form.PostForm, BaseTokenPage):

    ignoreContext = True
    invalidTokenMessage = _(u'Invalid Token.')
    invalidConsumerMessage = _(
        u'Consumer associated with this key is invalid.')
    deniedMessage = _(
        u'Token has been denied.')
    callbackInvalidMessage = _(
        u'Callback is not approved for the client.  Aborted.')
    token = None
    consumer = None
    description = ''
    verifier = ''
    statusTemplate = ViewPageTemplateFile(path('authorize_status.pt'))
    verifierTemplate = ViewPageTemplateFile(path('authorize_verifier.pt'))
    template = ViewPageTemplateFile(path('authorize_question.pt'))
    _errors = False

    def _checkConsumer(self, key):
        cm = zope.component.getMultiAdapter((self.context, self.request),
            IConsumerManager)

        consumer = cm.getValidated(key)
        if not consumer:
            raise ConsumerInvalidError('invalid consumer')
        return consumer

    def _checkToken(self, key):
        tm = zope.component.getMultiAdapter((self.context, self.request),
            ITokenManager)

        token = tm.get(key)
        if not token:
            raise TokenInvalidError('invalid token')
        return token

    def _update(self):
        token_key = self.request.form.get('oauth_token', None)
        token = self._checkToken(token_key)
        consumer = self._checkConsumer(token.consumer_key)
        self.token = token
        self.consumer = consumer

        sm = zope.component.getMultiAdapter((self.context, self.request),
            IScopeManager)
        view = zope.component.getMultiAdapter((sm, self.request),
            name='token_scope_view')
        view.omit_index = True
        try:
            self.scope = view()
        except NotFound:
            raise TokenInvalidError()

    def update(self):
        """\
        We do need an actual user, not sure which permission level will
        get me to do what I want in the zcml, hence we will need to
        manually check.
        """

        self.request['disable_border'] = True
        mt = getToolByName(self.context, 'portal_membership')
        if mt.isAnonymousUser():
            # should trigger a redirect to some login mechanism.
            raise Unauthorized()

        try:
            self._update()
        except TokenInvalidError, e:
            self._errors = self.invalidTokenMessage
        except ConsumerInvalidError, e:
            self._errors = self.invalidConsumerMessage

        if self._errors:
            self.status = self._errors
            self._errors = True

        return super(AuthorizeTokenForm, self).update() 

    def render(self):
        if self._errors:
            return self.statusTemplate()
        if self.verifier:
            return self.verifierTemplate()
        return super(AuthorizeTokenForm, self).render()

    def callback(self, verifier):
        if self.token.callback == 'oob':
            # Assign the verifier to this form to render for resource
            # owner's usage.
            self.verifier = verifier
            return True

        callback_url = self.token.get_callback_url()
        cbm = zope.component.getMultiAdapter((self.context, self.request),
            ICallbackManager)
        result = cbm.validate(self.consumer, callback_url)

        if result:
            # XXX what to do with the trusted parameter introduced in
            # between zope.publisher.http.HTTPRequest vs.
            # ZPublisher.HTTPResponse.HTTPResponse, as the latter does
            # not deal with that, and if this is in repoze, without
            # that redirection will not work.
            self.request.response.redirect(callback_url)
        else:
            # Abort; somehow the request token was approved prematurely
            # despite callback mismatch; perhaps the conditions have
            # changed?
            self._errors = True
            self.status = self.callbackInvalidMessage

        return result

    @button.buttonAndHandler(_('Grant access'), name='approve')
    def handleApprove(self, action):
        """\
        User approves this token.
        
        Redirect user to the callback URL to give the provider the OAuth
        Verifier key.
        """

        if self._errors or not self.token:
            return

        data, errors = self.extractData()

        mt = getToolByName(self.context, 'portal_membership')
        user = mt.getAuthenticatedMember().id

        tm = zope.component.getMultiAdapter((self.context, self.request),
            ITokenManager)
        if self.callback(self.token.verifier):
            tm.claimRequestToken(self.token, user)

    @button.buttonAndHandler(_('Deny access'), name='deny')
    def handleDeny(self, action):
        """\
        User denies this token
        """

        if self._errors or not self.token:
            return

        data, errors = self.extractData()

        token_key = self.request.form.get('oauth_token', None)
        tm = zope.component.getMultiAdapter((self.context, self.request),
            ITokenManager)
        tm.remove(token_key)
        sm = zope.component.getMultiAdapter((self.context, self.request),
            IScopeManager)
        sm.popScope(token_key, None)
        # we will let the client deal with rejection, if callback is
        # valid.
        self.callback(self.token.verifier)
        # if that wasn't valid, this is really the failure message.
        self.status = self.deniedMessage
        self._errors = True



from zope.annotation import factory as za_factory

def factory(cls, key=None):
    """
    This specialized annotation factory returns a factory that accepts
    the request attribute such that different layers can be applied to
    acquire the intended managers.
    """

    original = za_factory(cls, key)

    def getAnnotation(context, request):
        """\
        Call the original factory with just the context.
        """
        return original(context)

    return getAnnotation



import urllib
import urlparse
from time import time
from random import randint
from sys import maxint
from cStringIO import StringIO

from oauthlib.oauth1 import Client
from oauthlib.common import Request

import zope.interface
from zope.annotation.interfaces import IAttributeAnnotatable

from Testing import ZopeTestCase as ztc
from plone.session.tests.sessioncase import PloneSessionTestCase
from Zope2.App import zcml
from Products.Five import fiveconfigure
from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import PloneSite
from Products.PloneTestCase.layer import onsetup
from Products.PloneTestCase.layer import onteardown

import z3c.form.testing

import pmr2.z3cform.tests.base
from pmr2.oauth.plugins.oauth import OAuthPlugin

@onsetup
def setup():
    import pmr2.oauth
    fiveconfigure.debug_mode = True
    zcml.load_config('configure.zcml', pmr2.oauth)
    zcml.load_config('tests.zcml', pmr2.oauth.tests)
    fiveconfigure.debug_mode = False
    ztc.installPackage('pmr2.oauth')

@onteardown
def teardown():
    pass

setup()
teardown()
ptc.setupPloneSite(products=('pmr2.oauth',))


class OAuthTestCase(PloneSessionTestCase):

    def afterSetUp(self):
        PloneSessionTestCase.afterSetUp(self)
        self.app.folder = self.folder

        if self.folder.pas.hasObject("oauth"):
            self.app.folder.pas._delObject("oauth")

        self.app.folder.pas._setObject("oauth", OAuthPlugin("oauth"))


class IOAuthTestLayer(zope.interface.Interface):
    """\
    Mock layer
    """


def escape(s):
    return urllib.quote(s.encode('utf-8'), safe='~')


class TestRequest(pmr2.z3cform.tests.base.TestRequest):

    zope.interface.implements(IOAuthTestLayer, IAttributeAnnotatable)

    def __init__(self, oauth_keys=None, url=None, *a, **kw):
        super(TestRequest, self).__init__(*a, **kw)
        if url:
            parts = url.split('/')
            self._app_server = '/'.join(parts[:3])
            self._app_names = parts[3:]

        url = self.getURL()
        self.other = {}
        # Actual classes look for this
        self.other['ACTUAL_URL'] = url
        # Some other way of accessing this...
        self._environ['ACTUAL_URL'] = url

        if oauth_keys:
            self._auth = self.to_header(oauth_keys)

        self.stdin = StringIO()

    def to_header(self, oauth_keys, realm=''):
        # copied from oauth2 (for now)
        oauth_params = ((k, v) for k, v in oauth_keys.items()
                            if k.startswith('oauth_'))
        stringy_params = ((k, escape(str(v))) for k, v in oauth_params)
        header_params = ('%s="%s"' % (k, v) for k, v in stringy_params)
        params_header = ', '.join(header_params)

        auth_header = 'OAuth realm="%s"' % realm
        if params_header:
            auth_header = "%s, %s" % (auth_header, params_header)

        return auth_header


def SignedTestRequest(form=None, consumer=None, token=None, method=None,
        url=None, callback=None, timestamp=None, verifier=None, *a, **kw):
    """\
    Creates a signed TestRequest
    """

    def safe_unicode(s):
        # I really really hate oauthlib's insistence on using unicode
        # on things that are really bytes.
        if isinstance(s, str):
            return unicode(s)

        return s

    if not consumer:
        raise ValueError('consumer must be provided to build a signed request')

    if form is None:
        form = {}

    result = TestRequest(form=form, *a, **kw)
    url = url or result.getURL()
    url = safe_unicode(url)
    method = method and safe_unicode(method) or safe_unicode(result.method)
    timestamp = timestamp or unicode(int(time()))

    token_key = token and token.key
    token_secret = token and token.secret

    client = Client(
        safe_unicode(consumer.key),
        safe_unicode(consumer.secret),
        safe_unicode(token_key),
        safe_unicode(token_secret),
        safe_unicode(callback),
        verifier=safe_unicode(verifier),
        timestamp=timestamp,
    )

    url, headers, body = client.sign(url, method)

    result._auth = headers['Authorization']

    return result

def makeToken(qsstr):
    # quick and dirty, don't do this for real.
    d = urlparse.parse_qs(qsstr)
    t = {}
    t['key'] = d.get('oauth_token', ['']).pop()
    t['secret'] = d.get('oauth_token_secret', ['']).pop()
    return type('Token', (object,), t)



import zope.interface

import Acquisition

from pmr2.oauth.interfaces import IConsumer
from pmr2.oauth.interfaces import IConsumerManager
from pmr2.oauth.tests.base import TestRequest


class MockPAS(Acquisition.Implicit):
    def __init__(self):
        self.REQUEST = TestRequest()


class MockSite(Acquisition.Implicit):

    def absolute_url(self):
        return "http://nohost/"



import unittest

import zope.component

from zExceptions import Unauthorized
from Products.PloneTestCase import ptc

from pmr2.oauth.interfaces import ITokenManager, IConsumerManager
from pmr2.oauth.interfaces import IScopeManager
from pmr2.oauth.token import Token
from pmr2.oauth.consumer import Consumer
from pmr2.oauth.browser import consumer
from pmr2.oauth.browser import token
from pmr2.oauth.browser import user

from pmr2.oauth.tests.base import TestRequest


class FormTestCase(ptc.PloneTestCase):
    """
    Testing functionalities of forms that don't fit well into doctests.
    """

    def afterSetUp(self):
        request = TestRequest()
        self.consumerManager = zope.component.getMultiAdapter(
            (self.portal, request), IConsumerManager)
        consumer = Consumer('consumer.example.com', 'consumer-secret')
        self.consumerManager.add(consumer)

        self.tokenManager = zope.component.getMultiAdapter(
            (self.portal, request), ITokenManager)

        self.scopeManager = zope.component.getMultiAdapter(
            (self.portal, request), IScopeManager)

        self.reqtoken = self.tokenManager.generateRequestToken(
            consumer.key, 'oob')
        self.scopeManager.requestScope(self.reqtoken.key, None)

    def test_0000_authform_render(self):
        request = TestRequest(form={
            'oauth_token': self.reqtoken.key,
        })
        form = token.AuthorizeTokenForm(self.portal, request)
        form.update()
        result = form.render()
        self.assertTrue('_authenticator' in result)

    def test_0001_authform_post_authfail(self):
        request = TestRequest(form={
            'oauth_token': self.reqtoken.key,
            'form.buttons.approve': 1,
        })
        # simulate lack of CSRF
        request.form['_authenticator'] = None
        form = token.AuthorizeTokenForm(self.portal, request)
        self.assertRaises(Unauthorized, form.update)

    def test_0002_authform_post_authgood(self):
        request = TestRequest(form={
            'oauth_token': self.reqtoken.key,
            'form.buttons.approve': 1,
        })
        form = token.AuthorizeTokenForm(self.portal, request)
        form.update()
        result = form.render()
        self.assertTrue(self.reqtoken.verifier in result)

    def test_1000_consumermanageform_fail(self):
        request = TestRequest(form={
            'form.buttons.remove': 1,
        })
        request.form['_authenticator'] = None
        form = consumer.ConsumerManageForm(self.portal, request)
        self.assertRaises(Unauthorized, form.update)

    def test_2000_usertokenform_fail(self):
        request = TestRequest(form={
            'form.buttons.revoke': 1,
        })
        request.form['_authenticator'] = None
        form = user.UserTokenForm(self.portal, request)
        self.assertRaises(Unauthorized, form.update)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(FormTestCase))
    return suite



import new

import zope.interface
from zope.schema import Object, List
from zope.schema.interfaces import IList, ISequence

from pmr2.oauth.interfaces import _IDynamicSchemaInterface


class ICTSMMappingList(IList):
    pass


class ISchemaMethodObject(zope.interface.Interface):
    pass


class SchemaMethodObject(Object):
    """
    This is a mock object wrapper around a dict to provide just the
    schema interface.
    """

    zope.interface.implements(ISchemaMethodObject)

    @property
    def schema(self):
        # XXX this can be extremely heavy, will need to figure out how
        # to cache this.
        return self._schema()

    def __init__(self, schema, **kw):
        # The schema will be callable instead.
        self._schema = schema
        # Skip the parent validation.
        super(Object, self).__init__(**kw)

    def _validate(self, value):
        super(Object, self)._validate(value)
        # skip the schema validation as value is a dict.


class CTSMMappingList(List):
    """
    Customized List schema to prevent accidental overriding.
    """

    zope.interface.implements(ICTSMMappingList)


def buildSchemaInterface(fields, schema_factory, schema_keywords=None,
        iname='IDynamicSchemaInterface', sort=True,
    ):
    """
    Dynamic schema interface builder.
    """

    # Dragons in this method.

    default = {
        '__module__': __name__,
        '__doc__': 'Dynamic schema interface.',
    }

    items = enumerate(fields)
    if sort:
        # maybe check whether sort is callable, if so, use it as the
        # sort method.
        items = enumerate(sorted(fields))

    results = {}
    for c, values in items:
        id_, title = values

        kw = {
            'id': id_,
            'title': title,
            'required': False,
        } 

        if schema_keywords:
            kw.update(schema_keywords)

        field = schema_factory(**kw)
        # Specify the order, since the dict is orderless.
        field.order = c
        results[id_] = field
        
    default.update(results)
    
    # Incoming lion, get in the car.
    interfaceClass = new.classobj(iname, (_IDynamicSchemaInterface,), default)
        
    return interfaceClass



from setuptools import setup, find_packages
import os

version = '0.4.4'

setup(name='pmr2.oauth',
      version=version,
      description="OAuth PAS Plugin",
      long_description=open("README.rst").read() + "\n" +
           open(os.path.join("pmr2", "oauth", "README.rst")).read() + '\n' +
           open(os.path.join("docs", "UPGRADE.rst")).read() + '\n' +
           open(os.path.join("docs", "HISTORY.rst")).read(), 
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python",
        "Framework :: Plone :: 4.0",
        "Framework :: Plone :: 4.1",
        "Framework :: Plone :: 4.2",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Programming Language :: Python :: 2.5",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='',
      author='Tommy Yu',
      author_email='tommy.yu@auckland.ac.nz',
      url='https://github.com/PMR2/pmr2.oauth',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['pmr2'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'oauthlib==0.4.0',
          'zope.testing',
          'zope.component',
          'zope.interface',
          'zope.schema',
          'zope.annotation',
          'pmr2.z3cform',
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



from persistent import Persistent
from BTrees.OOBTree import OOBTree

from zope.app.container.contained import Contained
from zope.annotation.interfaces import IAttributeAnnotatable
import zope.interface
from zope.schema import fieldproperty

from pmr2.oauth.interfaces import IConsumer
from pmr2.oauth.interfaces import IConsumerManager
from pmr2.oauth.factory import factory
from pmr2.oauth.utility import random_string


class ConsumerManager(Persistent, Contained):
    """\
    A very basic consumer manager for the default layer.

    This manager only capture the very basics, and does not really
    allow users to add their own consumers and have them approved in 
    a way that is more integrated into the Plone (or other) CMS.
    """

    zope.component.adapts(IAttributeAnnotatable, zope.interface.Interface)
    zope.interface.implements(IConsumerManager)

    __dummy_key = fieldproperty.FieldProperty(IConsumer['key'])
    __dummy_secret = fieldproperty.FieldProperty(IConsumer['secret'])

    @property
    def DUMMY_KEY(self):
        return self.__dummy_key

    @property
    def DUMMY_SECRET(self):
        return self.__dummy_secret
    
    def __init__(self):
        self.__dummy_key = random_string(24)
        self.__dummy_secret = random_string(24)
        self._consumers = OOBTree()

    def add(self, consumer):
        assert IConsumer.providedBy(consumer)
        if self.get(consumer.key):
            raise ValueError('consumer %s already exists', consumer.key)
        self._consumers[consumer.key] = consumer

    def get(self, consumer_key, default=None):
        return self._consumers.get(consumer_key, default)

    def getValidated(self, consumer_key, default=None):
        # Provision for further checks by alternative implementations.
        return self.get(consumer_key, default)

    def getAllKeys(self):
        return self._consumers.keys()

    def makeDummy(self):
        return Consumer(str(self.DUMMY_KEY), str(self.DUMMY_SECRET))

    def remove(self, consumer):
        if IConsumer.providedBy(consumer):
            consumer = consumer.key
        self._consumers.pop(consumer)

ConsumerManagerFactory = factory(ConsumerManager)


class Consumer(Persistent):
    """\
    Basic persistent consumer class.
    """

    zope.interface.implements(IConsumer)

    key = fieldproperty.FieldProperty(IConsumer['key'])
    secret = fieldproperty.FieldProperty(IConsumer['secret'])
    title = fieldproperty.FieldProperty(IConsumer['title'])
    domain = fieldproperty.FieldProperty(IConsumer['domain'])

    def __init__(self, key, secret, title=None, domain=None):
        assert not ((key is None) or (secret is None))
        self.key = key
        self.secret = secret
        self.title = title
        self.domain = domain

    def __eq__(self, other):
        same_type = isinstance(other, self.__class__)
        return (same_type and 
            self.key == other.key and self.secret == other.secret)

    def validate(self):
        return True



import zope.component

from z3c.form.interfaces import IObjectWidget
from z3c.form.object import ObjectConverter

from pmr2.oauth.schema import ISchemaMethodObject


class SchemaMethodObjectConverter(ObjectConverter):
    """
    Data converter for the dynamic schema object that's really a 
    wrapper for a dict.
    """

    zope.component.adapts(ISchemaMethodObject, IObjectWidget)

    def toFieldValue(self, value):
        # The value provided is a dict, and this isn't an for an object
        # but borrows whatever useful from the object related classes to
        # hammer the desired value into the source dict, so the value is
        # already what is needed.
        return value


def NullConverter(obj):
    """
    Does not actually convert anything, just tell the adapter to reuse
    the object.
    """

    return obj



import zope.component
import zope.interface
from zope.publisher.browser import BrowserPage

from z3c.form import field
from z3c.form import button

from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage

from pmr2.z3cform import form

from pmr2.oauth import MessageFactory as _
from pmr2.oauth.interfaces import *
from pmr2.oauth.browser.template import ViewPageTemplateFile
from pmr2.oauth.browser.template import path
from pmr2.oauth.utility import random_string

from pmr2.oauth.consumer import Consumer


class ConsumerAddForm(form.AddForm):
    """\
    For adding consumers
    """

    fields = field.Fields(IConsumer).select(
        # key should be generated.
        # secret should be generated.
        'title',
        'domain',
    )

    def update(self):
        super(ConsumerAddForm, self).update()
        self.request['disable_border'] = True

    def create(self, data):
        # I don't think we need a consumer factory for this... just
        # inherit/redefine this form for your consumers.
        data['key'] = key = random_string(24)
        data['secret'] = secret = random_string(24)
        self._data = data
        return Consumer(key, secret, data['title'], data['domain'])

    def add(self, obj):
        cm = zope.component.getMultiAdapter(
            (self.context, self.request), IConsumerManager)
        cm.add(obj)

    def nextURL(self):
        return self.context.absolute_url() + '/manage-oauth-consumers'


class ConsumerManageForm(form.PostForm):
    """\
    For user to manage their authorized tokens.
    """

    ignoreContext = True
    template = ViewPageTemplateFile(path('consumer_manage_token.pt'))

    def getConsumers(self):
        cm = zope.component.getMultiAdapter((self.context, self.request),
            IConsumerManager)
        keys = cm.getAllKeys()

        # TODO rather than listing all secrets with this form, make it
        # so it will be possible to review the keys on a per-consumer
        # basis along with any fields specific to one.

        consumers = [cm.get(k) for k in keys]
        return consumers

    def update(self):
        super(ConsumerManageForm, self).update()
        self.consumers = self.getConsumers()
        self.request['disable_border'] = True

    @button.buttonAndHandler(_('Remove'), name='remove')
    def handleRemove(self, action):
        """\
        User revokes selected consumers.
        """

        # manually do everything since we are not using the built-in
        # widgets
        # TODO build/use widgets?
        # removing consumers does not remove corresponding tokens that 
        # were issued previous to this, although the tokens will cease
        # to work without the corresponding secret.
        data, errors = self.extractData()

        removed = error = 0
        keys = self.request.form.get('form.widgets.key', [])
        if isinstance(keys, basestring):
            # don't cast a string into a list as we are expecting one.
            keys = [keys]
        
        cm = zope.component.getMultiAdapter((self.context, self.request),
            IConsumerManager)
        for k in keys:
            try:
                cm.remove(k)
                removed += 1
            except:
                error = 1

        status = IStatusMessage(self.request)
        if error:
            status.addStatusMessage(
                _(u'Errors encountered during key removal'),
                type="error")
        if removed:
            status.addStatusMessage(
                _(u'Consumers successfully removed'),
                type="info")



import time
import unittest

import zope.component

from pmr2.oauth.consumer import ConsumerManager
from pmr2.oauth.consumer import Consumer

from pmr2.oauth.token import TokenManager
from pmr2.oauth.token import Token

from pmr2.oauth.interfaces import *

from pmr2.oauth.tests.base import TestRequest


class TestConsumer(unittest.TestCase):

    def setUp(self):
        pass

    def test_000_consumer(self):
        consumer = Consumer('consumer-key', 'consumer-secret')
        self.assertEqual(consumer.key, 'consumer-key')
        self.assertEqual(consumer.secret, 'consumer-secret')
        # Default should be no problem.
        self.assertTrue(consumer.validate())

        consumer = Consumer('consumer2-key', 'consumer-secret', u'A Consumer')
        self.assertEqual(consumer.key, 'consumer2-key')
        self.assertEqual(consumer.secret, 'consumer-secret')
        self.assertEqual(consumer.title, u'A Consumer')

    def test_001_consumer_equality(self):
        consumer = Consumer('consumer-key', 'consumer-secret')
        d = Consumer('consumer-key', 'consumer-secret', u'Test')
        # Titles don't factor into equality in terms of authorization
        # schemes.
        self.assertEqual(consumer, d)

    def test_100_consumer_manager_empty(self):
        m = ConsumerManager()
        self.assertEqual(m.get('consumer-key'), None)

    def test_101_consumer_manager_addget(self):
        m = ConsumerManager()
        consumer = Consumer('consumer-key', 'consumer-secret')
        m.add(consumer)
        result = m.get('consumer-key')
        self.assertEqual(result, consumer)

        consumer2 = Consumer('consumer2-key', 'consumer-secret', u'A Consumer')
        m.add(consumer2)
        result = m.get('consumer2-key')
        self.assertEqual(result, consumer2)
        self.assertEqual(result.title, consumer2.title)

    def test_102_consumer_manager_doubleadd(self):
        m = ConsumerManager()
        consumer = Consumer('consumer-key', 'consumer-secret')
        m.add(consumer)
        self.assertRaises(ValueError, m.add, consumer)

    def test_103_consumer_manager_remove(self):
        m = ConsumerManager()
        c1 = Consumer('consumer-key', 'consumer-secret')
        c2 = Consumer('consumer-key2', 'consumer-secret')
        m.add(c1)
        m.add(c2)
        m.remove(c1.key)
        m.remove(c2)
        self.assertEqual(len(m._consumers), 0)


class TestToken(unittest.TestCase):

    def setUp(self):
        pass
        #self.manager = ConsumerManager()

    def test_000_token(self):
        token = Token('token-key', 'token-secret')
        self.assertEqual(token.key, 'token-key')
        self.assertEqual(token.secret, 'token-secret')

    def test_010_token_set_verifier(self):
        token = Token('token-key', 'token-secret')
        token.set_verifier()
        verifier = token.verifier
        token.set_verifier()
        self.assertEqual(verifier, token.verifier)
        token.set_verifier(True)
        self.assertNotEqual(verifier, token.verifier)

    def test_011_token_set_verifier_specific(self):
        token = Token('token-key', 'token-secret')
        token.set_verifier('verify')
        self.assertEqual('verify', token.verifier)
        token.set_verifier()
        self.assertEqual('verify', token.verifier)

    def test_020_token_get_callback_url(self):
        token = Token('token-key', 'token-secret')
        token.set_callback('http://example.com/')
        token.set_verifier('foo')
        url = token.get_callback_url()
        a = 'http://example.com/?oauth_verifier=foo&oauth_token=token-key'
        self.assertEqual(url, a)

    def test_021_token_get_callback_url(self):
        token = Token('token-key', 'token-secret')
        token.set_callback('http://example.com/;bar;?bus=4')
        token.set_verifier('foo')
        url = token.get_callback_url()
        a = 'http://example.com/;bar;?bus=4&oauth_verifier=foo&' \
            'oauth_token=token-key'
        self.assertEqual(url, a)

    def test_100_token_manager_empty(self):
        m = TokenManager()
        self.assertEqual(m.get('token-key'), None)

    def test_101_token_manager_addget(self):
        m = TokenManager()
        token = Token('token-key', 'token-secret')
        m.add(token)
        result = m.get('token-key')
        self.assertEqual(result, token)
        result = m.get(token)
        self.assertEqual(result, token)

    def test_102_token_manager_doubleadd(self):
        m = TokenManager()
        token = Token('token-key', 'token-secret')
        m.add(token)
        self.assertRaises(ValueError, m.add, token)

    def test_103_token_manager_remove(self):
        m = TokenManager()
        t1 = Token('token-key', 'token-secret')
        t2 = Token('token-key2', 'token-secret')
        m.add(t1)
        m.add(t2)
        m.remove(t1.key)
        m.remove(t2)
        self.assertEqual(len(m._tokens), 1)

    def test_111_token_manager_addget_user(self):
        m = TokenManager()
        token = Token('token-key', 'token-secret')
        token.user = 'user'
        m.add(token)
        result = m.getTokensForUser('user')
        # only access tokens are tracked.
        self.assertEqual(result, [])

    def test_111_token_manager_addget_user(self):
        m = TokenManager()
        token = Token('token-key', 'token-secret')
        token.user = 'user'
        token.access = True
        m.add(token)
        result = m.getTokensForUser('user')
        self.assertEqual(result, [token])

    def test_113_token_manager_doubleadd_user(self):
        m = TokenManager()
        token = Token('token-key', 'token-secret')
        token.user = 'user'
        token.access = True
        m.add(token)
        self.assertRaises(ValueError, m.add, token)
        result = m.getTokensForUser('user')
        # should not result in double entry.
        self.assertEqual(result, [token])

    def test_114_token_manager_addremove_user(self):
        m = TokenManager()
        t1 = Token('token-key', 'token-secret')
        t1.user = 't1user'
        t1.access = True
        t2 = Token('token-key2', 'token-secret')
        t2.user = 't2user'
        t2.access = True
        m.add(t1)
        m.add(t2)
        self.assertEqual(m.getTokensForUser('user'), [])
        self.assertEqual(m.getTokensForUser('t1user'), [t1])
        self.assertEqual(m.getTokensForUser('t2user'), [t2])
        m.remove(t1.key)
        m.remove(t2)
        self.assertEqual(m.getTokensForUser('t1user'), [])
        self.assertEqual(m.getTokensForUser('t2user'), [])

    def test_115_token_manager_user_inconsistency(self):
        m = TokenManager()
        t1 = Token('token-key', 'token-secret')
        t1.user = 't1user'
        t1.access = True
        t2 = Token('token-key2', 'token-secret')
        t2.user = 't2user'
        t2.access = True
        m.add(t1)
        m.add(t2)
        m._del_user_map(t2)
        # User must know about the token for the getter to work.
        self.assertEqual(m.getTokensForUser('t2user'), [])

    def test_120_token_manager_access_token_tm_empty(self):
        m = TokenManager()
        self.assertRaises(TokenInvalidError, m.getAccessToken, 'token-key')
        self.assertRaises(TokenInvalidError, m.getRequestToken, 'token-key')
        self.assertEqual(m.getAccessToken('token-key', None), None)

    def test_121_token_manager_access_token_get_not_access(self):
        m = TokenManager()
        token = Token('token-key', 'token-secret')
        m.add(token)
        self.assertRaises(NotAccessTokenError, m.getAccessToken, 'token-key')
        self.assertEqual(m.getRequestToken('token-key'), token)

    def test_122_token_manager_access_token_get_user_no_access(self):
        m = TokenManager()
        token = Token('token-key', 'token-secret')
        token.user = 'user'
        m.add(token)
        self.assertRaises(NotAccessTokenError, m.getAccessToken, 'token-key')
        self.assertEqual(m.getRequestToken('token-key'), token)

    def test_123_token_manager_access_token_get_access_no_user(self):
        m = TokenManager()
        token = Token('token-key', 'token-secret')
        token.access = True
        m.add(token)
        self.assertRaises(TokenInvalidError, m.getAccessToken, 'token-key')
        self.assertRaises(NotRequestTokenError, m.getRequestToken, 'token-key')

    def test_124_token_manager_access_token_get_access(self):
        m = TokenManager()
        token = Token('token-key', 'token-secret')
        token.access = True
        token.user = 'user'
        m.add(token)
        self.assertEqual(m.getAccessToken('token-key'), token)
        self.assertRaises(NotRequestTokenError, m.getRequestToken, 'token-key')

    def test_125_token_manager_access_token_inconsistent_fail(self):
        m = TokenManager()
        token = Token('token-key', 'token-secret')
        token.access = True
        m.add(token)
        # this would result in the token not being indexed.
        token.user = 'user'
        self.assertRaises(TokenInvalidError, m.getAccessToken, 'token-key')
        self.assertRaises(NotRequestTokenError, m.getRequestToken, 'token-key')

    def test_200_token_manager_generate_request_token(self):
        m = TokenManager()
        consumer = Consumer('consumer-key', 'consumer-secret')
        callback = 'oob'
        token = m.generateRequestToken(consumer.key, callback)
        self.assertEqual(m.get(token.key), token)
        self.assertEqual(m.get(token.key).consumer_key, consumer.key)
        self.assertEqual(m.get(token.key).access, False)
        self.assertEqual(m.getRequestToken(token.key).key, token.key)

    def test_201_token_manager_generate_request_token_no_callback(self):
        m = TokenManager()
        consumer = Consumer('consumer-key', 'consumer-secret')
        self.assertRaises(CallbackValueError, m.generateRequestToken, 
            consumer.key, None)

    def test_250_token_manager_claim(self):
        m = TokenManager()
        consumer = Consumer('consumer-key', 'consumer-secret')
        callback = 'oob'
        token = m.generateRequestToken(consumer.key, callback)
        m.claimRequestToken(token, 'user')
        self.assertEqual(token.user, 'user')
        self.assertTrue(token.expiry > int(time.time()))

    def test_251_token_manager_claim_fail_access(self):
        m = TokenManager()
        consumer = Consumer('consumer-key', 'consumer-secret')
        callback = 'oob'
        token = m.generateRequestToken(consumer.key, callback)
        token.access = True  # hack it to be access token.
        self.assertRaises(TokenInvalidError, m.claimRequestToken, token, 'u')
        self.assertEqual(token.user, None)

    def test_252_token_manager_claim_fail_missing(self):
        m = TokenManager()
        consumer = Consumer('consumer-key', 'consumer-secret')
        callback = 'oob'
        token = m.generateRequestToken(consumer.key, callback)
        m.remove(token)  # remove it
        self.assertRaises(TokenInvalidError, m.claimRequestToken, token, 'u')
        self.assertEqual(token.user, None)

    def test_300_token_manager_generate_access_token(self):
        m = TokenManager()
        consumer = Consumer('consumer-key', 'consumer-secret')
        callback = 'oob'
        server_token = m.generateRequestToken(consumer.key, callback)
        verifier = server_token.verifier

        # Also simulate user claiming the token
        m.claimRequestToken(server_token.key, 'user')

        # now simulate passing only the key and secret to consumer
        request_token = Token(server_token.key, server_token.secret)
        token = m.generateAccessToken(consumer.key, request_token.key)

        self.assertEqual(m.get(token.key), token)
        self.assertEqual(m.get(token.key).consumer_key, consumer.key)
        self.assertEqual(m.get(token.key).access, True)
        self.assertEqual(m.get(token.key).user, 'user')

        # Also, assert that this token key is available in the method
        # that returns these keys by user id.
        self.assertEqual(m.getTokensForUser('user'), [token])

    def test_310_token_manager_generate_access_token_no_user(self):
        m = TokenManager()
        consumer = Consumer('consumer-key', 'consumer-secret')
        callback = 'oob'
        server_token = m.generateRequestToken(consumer.key, callback)

        # simulate passing only the key and secret to consumer
        request_token = Token(server_token.key, server_token.secret)

        # However, it's not claimed by a user yet, so expiry is not set,
        # thus...
        self.assertRaises(TokenInvalidError, m.generateAccessToken, 
            consumer.key, request_token.key)

    def test_311_token_manager_generate_access_token_no_request_token(self):
        m = TokenManager()
        consumer = Consumer('consumer-key', 'consumer-secret')
        self.assertRaises(TokenInvalidError, m.generateAccessToken, 
            consumer.key, None)

    def test_500_token_manager_get_dummy(self):
        m = TokenManager()
        token = m.get(m.DUMMY_KEY)
        self.assertEqual(token.secret, m.DUMMY_SECRET)
        # Now this is where it might get a bit interesting.  For dealing
        # with dummy tokens, is it better to generate a new one, or just
        # store it and provide one to ensure constant time?  Either way,
        # if somehow the dummy is removed, the utility shouldn't have to
        # die in a fire.
        m.remove(token)
        token = m.get(m.DUMMY_KEY)
        self.assertEqual(token, None)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestConsumer))
    suite.addTest(makeSuite(TestToken))
    return suite



import urlparse

from pmr2.oauth.callback import CallbackManager


class TestCallbackManager(CallbackManager):
    """
    Testing
    """

    permitted = ['nohost', 'localhost']

    def validate(self, consumer, callback):
        if super(TestCallbackManager, self).validate(consumer, callback):
            return True

        # Also permit localhost, nohost and other test hostnames.
        scheme, netloc = urlparse.urlparse(callback)[:2]
        return netloc in self.permitted



import zope.component
import zope.interface
from zope.publisher.browser import BrowserPage
from zope.app.component.hooks import getSite
from zope.publisher.interfaces import NotFound

from z3c.form import button

from Acquisition import Implicit
from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage

from pmr2.z3cform import form, page

from pmr2.oauth import MessageFactory as _
from pmr2.oauth.interfaces import IConsumerManager, ITokenManager
from pmr2.oauth.interfaces import IContentTypeScopeManager, IScopeManager
from pmr2.oauth.browser.template import ViewPageTemplateFile
from pmr2.oauth.browser.template import path
from pmr2.oauth.browser.scope import TokenCTScopeView


class _NotConsumer(object):
     title = _(u'<Deleted Client>')


class BaseUserTokenForm(form.PostForm):
    """\
    For user to manage their authorized tokens.
    """

    ignoreContext = True
    template = ViewPageTemplateFile(path('user_manage_token.pt'))

    @property
    def url_expr(self):
        # URL expression for this view.
        return '%s/%s' % (self.context.absolute_url(), self.__name__)

    def getUser(self):
        # Returns the current user, or the target user.
        raise NotImplemented

    def getTokens(self):
        user = self.getUser()
        cm = zope.component.getMultiAdapter((self.context, self.request),
            IConsumerManager)
        tm = zope.component.getMultiAdapter((self.context, self.request),
            ITokenManager)
        tokens = []
        for token in tm.getTokensForUser(user):
            consumer_title = cm.get(token.consumer_key, _NotConsumer).title
            tokens.append({
                'consumer_title': consumer_title,
                'key': token.key,
            })
        return tokens

    def update(self):
        super(BaseUserTokenForm, self).update()
        self.tokens = self.getTokens()
        self.request['disable_border'] = True

    def revokeTokens(self):
        """
        User revokes selected tokens.
        """

        # manually do everything since we are not using the built-in
        # widgets
        # TODO use widgets?

        current_user = self.getUser()
        valid = self.getTokens()
        removed = error = 0
        keys = self.request.form.get('form.widgets.key', [])
        if isinstance(keys, basestring):
            # don't cast a string into a list as we are expecting one.
            keys = [keys]
        
        tm = zope.component.getMultiAdapter((self.context, self.request),
            ITokenManager)
        sm = zope.component.getMultiAdapter((self.context, self.request),
            IScopeManager)

        for k in keys:
            token = tm.get(k)
            if token is None or not token.user == current_user:
                error = 1
                continue

            try:
                tm.remove(k)
                sm.delAccessScope(k, None)
                removed += 1
            except:
                error = 1

        status = IStatusMessage(self.request)
        if error:
            status.addStatusMessage(
                _(u'Errors encountered during key removal'),
                type="error")
        if removed:
            status.addStatusMessage(
                _(u'Access successfully removed'),
                type="info")


class UserTokenForm(BaseUserTokenForm):

    def getUser(self):
        mt = getToolByName(self.context, 'portal_membership')
        return mt.getAuthenticatedMember().id

    @button.buttonAndHandler(_('Revoke'), name='revoke')
    def handleRevoke(self, action):
        data, errors = self.extractData()
        return self.revokeTokens()


class UserTokenListView(Implicit, BrowserPage):

    def publishTraverse(self, request, name):
        # viewlet manager doesn't seem to like implicit objects, so this
        # and the above has to be split up...
        result = UserTokenForm(self.context, self.request)
        result.__name__ = self.__name__
        return result


class UserTokenCTScopeDetailsView(page.TraversePage, TokenCTScopeView):
    """
    This view class explicitly includes ContentTypeScope informations.
    Plugins that wish to override this scope manager will also need to
    override this view appropriately with the use of layers.
    """

    template = ViewPageTemplateFile(path('token_scope_view.pt'))

    def getTokenKey(self):
        if not self.traverse_subpath or len(self.traverse_subpath) > 1:
            return None
        return self.traverse_subpath[0]

    def getScope(self, token_key):
        site = getSite()
        sm = zope.component.getMultiAdapter(
            (site, self.request), IContentTypeScopeManager)
        # scopes for this manager is a set of mapping_ids.
        return sm.getAccessScope(token_key, None)

    def update(self):
        site = getSite()
        tm = zope.component.getMultiAdapter(
            (site, self.request), ITokenManager)
        cm = zope.component.getMultiAdapter(
            (site, self.request), IConsumerManager)
        token_key = self.getTokenKey()
        token = tm.getAccessToken(token_key, None)

        mt = getToolByName(self.context, 'portal_membership')
        current_user = mt.getAuthenticatedMember().id

        if token is None or not token.user == current_user:
            raise NotFound(self.context, token_key)

        self.consumer_title = cm.get(token.consumer_key, _NotConsumer).title

        # Set the scope attributes only after this token is found for
        # this user.
        super(UserTokenCTScopeDetailsView, self).update()
